# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_GTMPVC_METADATA = Metadata(
    id="fd930d51e0a8f5012076ae523ec1cb2238ba47c5.boutiques",
    name="mri_gtmpvc",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriGtmpvcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_gtmpvc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    synthesized_volume: OutputPathType
    """Synthesized volume"""
    rescaled_input: OutputPathType
    """Rescaled input volume"""
    eresiduals: OutputPathType
    """Residual errors volume"""
    yhat: OutputPathType
    """Yhat volume"""
    yhat_with_noise: OutputPathType
    """Yhat with noise volume"""
    yhat_full_fov: OutputPathType
    """Yhat full FoV volume"""
    yhat_prior: OutputPathType
    """Yhat prior to smoothing volume"""
    x_matrix: OutputPathType
    """X matrix in matlab4 format"""
    y_matrix: OutputPathType
    """Y matrix in matlab4 format"""
    beta_matrix: OutputPathType
    """Beta matrix in matlab4 format"""
    x0_matrix: OutputPathType
    """X0 matrix in matlab4 format"""
    gtm_values_text: OutputPathType
    """Demeaned GTM values as text file"""


def mri_gtmpvc(
    input_volume: InputPathType,
    psf: float,
    segmentation: InputPathType,
    output_directory: str,
    frame: float | None = None,
    registration: InputPathType | None = None,
    mask: InputPathType | None = None,
    auto_mask: int | None = None,
    no_reduce_fov: bool = False,
    reduce_fov_eqodd: bool = False,
    contrast_matrix: InputPathType | None = None,
    default_seg_merge: bool = False,
    merge_hypos: bool = False,
    merge_cblum_wm_gyri: bool = False,
    tt_reduce: bool = False,
    replace_seg: str | None = None,
    replace_file: InputPathType | None = None,
    rescale: str | None = None,
    no_rescale: bool = False,
    scale_refval: float | None = None,
    ctab: InputPathType | None = None,
    ctab_default: bool = False,
    tt_update: bool = False,
    lateralization: bool = False,
    no_tfe: bool = False,
    no_pvc: bool = False,
    segpvfres: float | None = None,
    rbv: bool = False,
    rbv_res: float | None = None,
    mueller_pvc: str | None = None,
    mg_ref_cerebral_wm: bool = False,
    mg_ref_lobes_wm: bool = False,
    glm_mg_pvc: float | None = None,
    km_ref: str | None = None,
    km_hb: str | None = None,
    steady_state: str | None = None,
    save_x: bool = False,
    save_y: bool = False,
    save_beta: bool = False,
    save_x0: bool = False,
    save_input: bool = False,
    save_eres: bool = False,
    save_yhat: bool = False,
    save_yhat_noise: str | None = None,
    save_yhat_full_fov: bool = False,
    save_yhat0: bool = False,
    synth: str | None = None,
    synth_only: bool = False,
    synth_save: bool = False,
    save_text: bool = False,
    threads: float | None = None,
    max_threads: bool = False,
    max_threads_minus_one: bool = False,
    subjects_dir: str | None = None,
    vg_thresh: float | None = None,
    gdiag: float | None = None,
    debug: bool = False,
    checkopts: bool = False,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MriGtmpvcOutputs:
    """
    mri_gtmpvc performs partial volume correction on PET data using anatomical
    segmentation.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: Input volume: source data to PVC.
        psf: Scanner PSF FWHM in mm.
        segmentation: Anatomical segmentation to define regions for GTM.
        output_directory: Output directory.
        frame: Only process 0-based frame F from input volume.
        registration: LTA registration file that maps PET to anatomical.
        mask: Ignore areas outside of the mask (in input vol space).
        auto_mask: Automatically compute mask with FWHM and threshold.
        no_reduce_fov: Do not reduce FoV to encompass mask.
        reduce_fov_eqodd: Reduce FoV to encompass mask but force nc=nr and ns\
            to be odd.
        contrast_matrix: Univariate contrast to test.
        default_seg_merge: Default schema for merging ROIs.
        merge_hypos: Merge left and right hypointensites into ROI.
        merge_cblum_wm_gyri: Cerebellum WM gyri back into cerebellum WM.
        tt_reduce: Reduce segmentation to that of a tissue type.
        replace_seg: Replace seg Id1 with seg Id2.
        replace_file: File with a list of Ids to replace.
        rescale: Specify reference region(s) used to rescale (default is pons).
        no_rescale: Do not global rescale such that mean of reference region is\
            scaleref.
        scale_refval: Scale such that mean in reference region is refval.
        ctab: Specify color table explicitly.
        ctab_default: Use default color table.
        tt_update: Changes tissue type of VentralDC, BrainStem, and Pons to be\
            SubcortGM.
        lateralization: Lateralize tissue types.
        no_tfe: Do not correct for tissue fraction effect.
        no_pvc: Turns off PVC entirely.
        segpvfres: Set the tissue fraction resolution parameter (default is\
            0.5).
        rbv: Perform RBV PVC.
        rbv_res: Set RBV voxel resolution.
        mueller_pvc: Perform Mueller-Gaertner PVC.
        mg_ref_cerebral_wm: Set MG RefIds to 2 and 41.
        mg_ref_lobes_wm: Set MG RefIds to those for lobes when using wm subseg.
        glm_mg_pvc: GLM-based Mueller-Gaertner PVC.
        km_ref: Compute reference TAC for KM as mean of given RefIds.
        km_hb: Compute HiBinding TAC for KM as mean of given RefIds.
        steady_state: Steady-state analysis spec blood plasma concentration,\
            unit scale, and decay correction factor.
        save_x: Save X matrix in matlab4 format as X.mat.
        save_y: Save y matrix in matlab4 format as y.mat.
        save_beta: Save beta matrix in matlab4 format as beta.mat.
        save_x0: Save X0 matrix in matlab4 format as X0.mat.
        save_input: Saves rescaled input as input.rescaled.nii.gz.
        save_eres: Saves residual error.
        save_yhat: Saves yhat.
        save_yhat_noise: Saves yhat with noise, seed < 0 for TOD.
        save_yhat_full_fov: Saves yhat in full FoV (if FoV was reduced).
        save_yhat0: Saves yhat prior to smoothing.
        synth: Synthesize volume with gtmbeta as input.
        synth_only: Exit after doing synthesis (implies --synth-save).
        synth_save: With --synth saves synthesized volume to\
            outdir/synth.nii.gz.
        save_text: Save demeaned GTM values out to text files named after the\
            seg.
        threads: Use N threads (with Open MP).
        max_threads: Use the maximum allowable number of threads for this\
            computer.
        max_threads_minus_one: Use one less than the maximum allowable number\
            of threads for this computer.
        subjects_dir: Specify SUBJECTS_DIR.
        vg_thresh: Threshold for LTAconcat error.
        gdiag: Set diagnostic level.
        debug: Turn on debugging.
        checkopts: Don't run anything, just check options and exit.
        help_: Print out information on how to use this program.
        version: Print out version and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriGtmpvcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_GTMPVC_METADATA)
    cargs = []
    cargs.append("mri_gtmpvc")
    cargs.extend([
        "--i",
        execution.input_file(input_volume)
    ])
    if frame is not None:
        cargs.extend([
            "--frame",
            str(frame)
        ])
    cargs.extend([
        "--psf",
        str(psf)
    ])
    cargs.extend([
        "--seg",
        execution.input_file(segmentation)
    ])
    if registration is not None:
        cargs.extend([
            "--reg",
            execution.input_file(registration)
        ])
    cargs.extend([
        "--o",
        output_directory
    ])
    if mask is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask)
        ])
    if auto_mask is not None:
        cargs.extend([
            "--auto-mask",
            str(auto_mask)
        ])
    if no_reduce_fov:
        cargs.append("--no-reduce-fov")
    if reduce_fov_eqodd:
        cargs.append("--reduce-fov-eqodd")
    if contrast_matrix is not None:
        cargs.extend([
            "--C",
            execution.input_file(contrast_matrix)
        ])
    if default_seg_merge:
        cargs.append("--default-seg-merge")
    if merge_hypos:
        cargs.append("--merge-hypos")
    if merge_cblum_wm_gyri:
        cargs.append("--merge-cblum-wm-gyri")
    if tt_reduce:
        cargs.append("--tt-reduce")
    if replace_seg is not None:
        cargs.extend([
            "--replace",
            replace_seg
        ])
    if replace_file is not None:
        cargs.extend([
            "--replace-file",
            execution.input_file(replace_file)
        ])
    if rescale is not None:
        cargs.extend([
            "--rescale",
            rescale
        ])
    if no_rescale:
        cargs.append("--no-rescale")
    if scale_refval is not None:
        cargs.extend([
            "--scale-refval",
            str(scale_refval)
        ])
    if ctab is not None:
        cargs.extend([
            "--ctab",
            execution.input_file(ctab)
        ])
    if ctab_default:
        cargs.append("--ctab-default")
    if tt_update:
        cargs.append("--tt-update")
    if lateralization:
        cargs.append("--lat")
    if no_tfe:
        cargs.append("--no-tfe")
    if no_pvc:
        cargs.append("--no-pvc")
    if segpvfres is not None:
        cargs.extend([
            "--segpvfres",
            str(segpvfres)
        ])
    if rbv:
        cargs.append("--rbv")
    if rbv_res is not None:
        cargs.extend([
            "--rbv-res",
            str(rbv_res)
        ])
    if mueller_pvc is not None:
        cargs.extend([
            "--mg",
            mueller_pvc
        ])
    if mg_ref_cerebral_wm:
        cargs.append("--mg-ref-cerebral-wm")
    if mg_ref_lobes_wm:
        cargs.append("--mg-ref-lobes-wm")
    if glm_mg_pvc is not None:
        cargs.extend([
            "--mgx",
            str(glm_mg_pvc)
        ])
    if km_ref is not None:
        cargs.extend([
            "--km-ref",
            km_ref
        ])
    if km_hb is not None:
        cargs.extend([
            "--km-hb",
            km_hb
        ])
    if steady_state is not None:
        cargs.extend([
            "--ss",
            steady_state
        ])
    if save_x:
        cargs.append("--X")
    if save_y:
        cargs.append("--y")
    if save_beta:
        cargs.append("--beta")
    if save_x0:
        cargs.append("--X0")
    if save_input:
        cargs.append("--save-input")
    if save_eres:
        cargs.append("--save-eres")
    if save_yhat:
        cargs.append("--save-yhat")
    if save_yhat_noise is not None:
        cargs.extend([
            "--save-yhat-with-noise",
            save_yhat_noise
        ])
    if save_yhat_full_fov:
        cargs.append("--save-yhat-full-fov")
    if save_yhat0:
        cargs.append("--save-yhat0")
    if synth is not None:
        cargs.extend([
            "--synth",
            synth
        ])
    if synth_only:
        cargs.append("--synth-only")
    if synth_save:
        cargs.append("--synth-save")
    if save_text:
        cargs.append("--save-text")
    if threads is not None:
        cargs.extend([
            "--threads",
            str(threads)
        ])
    if max_threads:
        cargs.append("--max-threads")
    if max_threads_minus_one:
        cargs.append("--max-threads-minus-1")
    if subjects_dir is not None:
        cargs.extend([
            "--sd",
            subjects_dir
        ])
    if vg_thresh is not None:
        cargs.extend([
            "--vg-thresh",
            str(vg_thresh)
        ])
    if gdiag is not None:
        cargs.extend([
            "--gdiag",
            str(gdiag)
        ])
    if debug:
        cargs.append("--debug")
    if checkopts:
        cargs.append("--checkopts")
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    ret = MriGtmpvcOutputs(
        root=execution.output_file("."),
        synthesized_volume=execution.output_file(output_directory + "/synth.nii.gz"),
        rescaled_input=execution.output_file(output_directory + "/input.rescaled.nii.gz"),
        eresiduals=execution.output_file(output_directory + "/eresiduals.nii.gz"),
        yhat=execution.output_file(output_directory + "/yhat.nii.gz"),
        yhat_with_noise=execution.output_file(output_directory + "/yhat_with_noise.nii.gz"),
        yhat_full_fov=execution.output_file(output_directory + "/yhat_full_fov.nii.gz"),
        yhat_prior=execution.output_file(output_directory + "/yhat_prior.nii.gz"),
        x_matrix=execution.output_file(output_directory + "/X.mat"),
        y_matrix=execution.output_file(output_directory + "/y.mat"),
        beta_matrix=execution.output_file(output_directory + "/beta.mat"),
        x0_matrix=execution.output_file(output_directory + "/X0.mat"),
        gtm_values_text=execution.output_file(output_directory + "/gtm_values.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_GTMPVC_METADATA",
    "MriGtmpvcOutputs",
    "mri_gtmpvc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CONCATENATE_LTA_METADATA = Metadata(
    id="4ebf515f1f226122f61f4db868b19e295a774022.boutiques",
    name="mri_concatenate_lta",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriConcatenateLtaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_concatenate_lta(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_concatenate_lta(
    lta_1: InputPathType,
    lta_2: InputPathType,
    lta_final: str,
    tal_src: InputPathType | None = None,
    tal_template: InputPathType | None = None,
    invert1: bool = False,
    invert2: bool = False,
    invertout: bool = False,
    out_type: float | None = None,
    subject: str | None = None,
    rmsdiff_radius: float | None = None,
    rmsdiff_outputfile: str | None = None,
    runner: Runner | None = None,
) -> MriConcatenateLtaOutputs:
    """
    Concatenates two consecutive LTA transformations into one overall
    transformation, Out = LTA2*LTA1.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        lta_1: File for LTA transformation that maps some src1 to dst1.
        lta_2: File for LTA transformation that maps dst1 (src2) to dst2.
        lta_final: File for the combined LTA maps: src1 to dst2 = LTA2*LTA1.
        tal_src: Specify source (file1) for Talairach transformation.
        tal_template: Specify template (file2) for Talairach transformation.
        invert1: Invert LTA1 before applying it.
        invert2: Invert LTA2 before applying it.
        invertout: Invert output LTA.
        out_type: Set final LTA type: 0 for VOX2VOX (default), 1 for RAS2RAS.
        subject: Set subject in output LTA.
        rmsdiff_radius: Radius used for computing RMS diff between transforms.
        rmsdiff_outputfile: Output file for RMS diff computation. Use 'nofile'\
            to skip output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriConcatenateLtaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CONCATENATE_LTA_METADATA)
    cargs = []
    cargs.append("mri_concatenate_lta")
    cargs.append(execution.input_file(lta_1))
    cargs.append(execution.input_file(lta_2))
    cargs.append(lta_final)
    if tal_src is not None:
        cargs.extend([
            "-tal",
            execution.input_file(tal_src)
        ])
    if tal_template is not None:
        cargs.append(execution.input_file(tal_template))
    if invert1:
        cargs.append("-invert1")
    if invert2:
        cargs.append("-invert2")
    if invertout:
        cargs.append("-invertout")
    if out_type is not None:
        cargs.extend([
            "-out_type",
            str(out_type)
        ])
    if subject is not None:
        cargs.extend([
            "-subject",
            subject
        ])
    if rmsdiff_radius is not None:
        cargs.extend([
            "-rmsdiff",
            str(rmsdiff_radius)
        ])
    if rmsdiff_outputfile is not None:
        cargs.append(rmsdiff_outputfile)
    ret = MriConcatenateLtaOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CONCATENATE_LTA_METADATA",
    "MriConcatenateLtaOutputs",
    "mri_concatenate_lta",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

QDELAUNAY_METADATA = Metadata(
    id="6ffd24be5e49d65ad58c48581dbc1c38e779c71c.boutiques",
    name="qdelaunay",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class QdelaunayOutputs(typing.NamedTuple):
    """
    Output object returned when calling `qdelaunay(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def qdelaunay(
    input_file: InputPathType,
    furthest_site: bool = False,
    triangulated_output: bool = False,
    joggled_input: bool = False,
    joggle_range: float | None = None,
    search_simplex: bool = False,
    point_infinity: bool = False,
    delaunay_visible: str | None = None,
    delaunay_regions: str | None = None,
    trace_level: float | None = None,
    check: bool = False,
    statistics_: bool = False,
    verify: bool = False,
    output_stdout: bool = False,
    facets_summary: float | None = None,
    input_file_option: InputPathType | None = None,
    output_file_option: InputPathType | None = None,
    trace_point: float | None = None,
    trace_merge: float | None = None,
    trace_merge_width: float | None = None,
    stop_point: float | None = None,
    stop_cone_point: float | None = None,
    centrum_radius: float | None = None,
    max_angle_cosine: float | None = None,
    perturb_factor: float | None = None,
    min_facet_width: float | None = None,
    facet_dump: bool = False,
    geomview: bool = False,
    vertices_incident: bool = False,
    mathematica: bool = False,
    off_format: bool = False,
    point_coordinates: bool = False,
    summary: bool = False,
    runner: Runner | None = None,
) -> QdelaunayOutputs:
    """
    Compute the Delaunay triangulation using Qhull.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_file: Input file containing point coordinates.
        furthest_site: Compute furthest-site Delaunay triangulation.
        triangulated_output: Triangulated output.
        joggled_input: Joggled input instead of merged facets.
        joggle_range: Randomly joggle input in range [-n,n].
        search_simplex: Search all points for the initial simplex.
        point_infinity: Add point-at-infinity to Delaunay triangulation.
        delaunay_visible: Print Delaunay region if visible from point n, -n if\
            not.
        delaunay_regions: Print Delaunay regions that include point n, -n if\
            not.
        trace_level: Trace at level n, 4=all, 5=mem/gauss, -1= events.
        check: Check frequently during execution.
        statistics_: Print statistics.
        verify: Verify result: structure, convexity, and in-circle test.
        output_stdout: Send all output to stdout.
        facets_summary: Report summary when n or more facets created.
        input_file_option: Input data from file, no spaces or single quotes.
        output_file_option: Output results to file, may be enclosed in single\
            quotes.
        trace_point: Turn on tracing when point n added to hull.
        trace_merge: Turn on tracing at merge n.
        trace_merge_width: Trace merge facets when width > n.
        stop_point: Stop Qhull after adding point n, -n for before.
        stop_cone_point: Stop Qhull after building cone for point n.
        centrum_radius: Radius of centrum (roundoff added). Merge facets if\
            non-convex.
        max_angle_cosine: Cosine of maximum angle. Merge facets if cosine > n\
            or non-convex.
        perturb_factor: Randomly perturb computations by a factor of [1-n,1+n].
        min_facet_width: Min facet width for outside point (before roundoff).
        facet_dump: Facet dump.
        geomview: Geomview output.
        vertices_incident: Vertices incident to each Delaunay region.
        mathematica: Mathematica output (2-d only, lifted to a paraboloid).
        off_format: OFF format (dim, points, and facets as a paraboloid).
        point_coordinates: Point coordinates (lifted to a paraboloid).
        summary: Summary (stderr).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `QdelaunayOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(QDELAUNAY_METADATA)
    cargs = []
    cargs.append("qdelaunay")
    cargs.append(execution.input_file(input_file))
    if furthest_site:
        cargs.append("Qu")
    if triangulated_output:
        cargs.append("Qt")
    if joggled_input:
        cargs.append("QJ")
    if joggle_range is not None:
        cargs.extend([
            "QJn",
            str(joggle_range)
        ])
    if search_simplex:
        cargs.append("Qs")
    if point_infinity:
        cargs.append("Qz")
    if delaunay_visible is not None:
        cargs.extend([
            "QGn",
            delaunay_visible
        ])
    if delaunay_regions is not None:
        cargs.extend([
            "QVn",
            delaunay_regions
        ])
    if trace_level is not None:
        cargs.extend([
            "T4",
            str(trace_level)
        ])
    if check:
        cargs.append("Tc")
    if statistics_:
        cargs.append("Ts")
    if verify:
        cargs.append("Tv")
    if output_stdout:
        cargs.append("Tz")
    if facets_summary is not None:
        cargs.extend([
            "TFn",
            str(facets_summary)
        ])
    if input_file_option is not None:
        cargs.extend([
            "TI",
            execution.input_file(input_file_option)
        ])
    if output_file_option is not None:
        cargs.extend([
            "TO",
            execution.input_file(output_file_option)
        ])
    if trace_point is not None:
        cargs.extend([
            "TPn",
            str(trace_point)
        ])
    if trace_merge is not None:
        cargs.extend([
            "TMn",
            str(trace_merge)
        ])
    if trace_merge_width is not None:
        cargs.extend([
            "TWn",
            str(trace_merge_width)
        ])
    if stop_point is not None:
        cargs.extend([
            "TVn",
            str(stop_point)
        ])
    if stop_cone_point is not None:
        cargs.extend([
            "TCn",
            str(stop_cone_point)
        ])
    if centrum_radius is not None:
        cargs.extend([
            "Cn",
            str(centrum_radius)
        ])
    if max_angle_cosine is not None:
        cargs.extend([
            "An",
            str(max_angle_cosine)
        ])
    if perturb_factor is not None:
        cargs.extend([
            "Rn",
            str(perturb_factor)
        ])
    if min_facet_width is not None:
        cargs.extend([
            "Wn",
            str(min_facet_width)
        ])
    if facet_dump:
        cargs.append("f")
    if geomview:
        cargs.append("G")
    if vertices_incident:
        cargs.append("i")
    if mathematica:
        cargs.append("m")
    if off_format:
        cargs.append("o")
    if point_coordinates:
        cargs.append("p")
    if summary:
        cargs.append("s")
    ret = QdelaunayOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "QDELAUNAY_METADATA",
    "QdelaunayOutputs",
    "qdelaunay",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SLOW_SURF_CLUSTSIM_PY_METADATA = Metadata(
    id="a7e11fd0c89e1ec778cc2b10b112e4d443b843ac.boutiques",
    name="slow_surf_clustsim.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SlowSurfClustsimPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `slow_surf_clustsim_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def slow_surf_clustsim_py(
    on_surface: str | None = None,
    save_script: str | None = None,
    print_script: bool = False,
    uvar: list[str] | None = None,
    verbosity: float | None = None,
    help_: bool = False,
    hist: bool = False,
    show_default_cvars: bool = False,
    show_default_uvars: bool = False,
    show_valid_opts: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> SlowSurfClustsimPyOutputs:
    """
    Generate a tcsh script to run clustsim on surface.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        on_surface: Start from noise on the surface (so no volume data is\
            involved).
        save_script: Save script to given file.
        print_script: Print script to terminal.
        uvar: Set the user variable (use -show_default_uvars to see user vars).\
            Example usage: -uvar spec_file sb23_lh_141_std.spec -uvar surf_vol\
            sb23_SurfVol_aligned+orig.
        verbosity: Set the verbosity level.
        help_: Show this help.
        hist: Show module history.
        show_default_cvars: List default control variables.
        show_default_uvars: List default user variables.
        show_valid_opts: List valid options.
        version: Show current version.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SlowSurfClustsimPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SLOW_SURF_CLUSTSIM_PY_METADATA)
    cargs = []
    cargs.append("slow_surf_clustsim.py")
    if on_surface is not None:
        cargs.extend([
            "-on_surface",
            on_surface
        ])
    if save_script is not None:
        cargs.extend([
            "-save_script",
            save_script
        ])
    if print_script:
        cargs.append("-print_script")
    if uvar is not None:
        cargs.extend([
            "-uvar",
            *uvar
        ])
    if verbosity is not None:
        cargs.extend([
            "-verb",
            str(verbosity)
        ])
    if help_:
        cargs.append("-help")
    if hist:
        cargs.append("-hist")
    if show_default_cvars:
        cargs.append("-show_default_cvars")
    if show_default_uvars:
        cargs.append("-show_default_uvars")
    if show_valid_opts:
        cargs.append("-show_valid_opts")
    if version:
        cargs.append("-ver")
    ret = SlowSurfClustsimPyOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SLOW_SURF_CLUSTSIM_PY_METADATA",
    "SlowSurfClustsimPyOutputs",
    "slow_surf_clustsim_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__VOL_CENTER_METADATA = Metadata(
    id="4ce13662130110b3fd50b4a405a5d296faf47380.boutiques",
    name="@VolCenter",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VVolCenterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__vol_center(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__vol_center(
    dset: InputPathType,
    orient: str | None = None,
    runner: Runner | None = None,
) -> VVolCenterOutputs:
    """
    Tool to return the center of volume for a given dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dset: Input volume dataset.
        orient: Output coordinate system orientation (e.g., RAI).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VVolCenterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__VOL_CENTER_METADATA)
    cargs = []
    cargs.append("@VolCenter")
    cargs.extend([
        "-dset",
        execution.input_file(dset)
    ])
    if orient is not None:
        cargs.extend([
            "-or",
            orient
        ])
    ret = VVolCenterOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VVolCenterOutputs",
    "V__VOL_CENTER_METADATA",
    "v__vol_center",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_ATROPOS_N4_SH_METADATA = Metadata(
    id="26f6c01fef89c147f9f9fc215073f4d86231d70b.boutiques",
    name="antsAtroposN4.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


@dataclasses.dataclass
class AntsAtroposN4ShSegmentationPriors:
    """
    Prior probability images initializing the segmentation. Specified using
    c-style formatting, e.g. -p labelsPriors%02d.nii.gz. If this is not
    specified, k-means initialization is used instead.
    """
    segmentation_priors_pattern: str | None = None
    """Prior probability images initializing the segmentation. Specified using
    c-style formatting, e.g. -p labelsPriors%02d.nii.gz. If this is not
    specified, k-means initialization is used instead."""
    segmentation_priors_folder: InputPathType | None = None
    """Included so."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.segmentation_priors_pattern is not None or self.segmentation_priors_folder is not None:
            cargs.append((self.segmentation_priors_pattern if (self.segmentation_priors_pattern is not None) else "") + "/" + (execution.input_file(self.segmentation_priors_folder) if (self.segmentation_priors_folder is not None) else ""))
        return cargs


class AntsAtroposN4ShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_atropos_n4_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    n4_corrected: OutputPathType
    """N4 corrected image."""
    segmentation: OutputPathType
    """Segmentation image."""
    segmentation_posteriors: OutputPathType
    """Segmentation posteriors image."""


def ants_atropos_n4_sh(
    image_dimension: typing.Literal[2, 3],
    input_image: InputPathType,
    mask_image: InputPathType,
    number_of_classes: int,
    output_prefix: str,
    segmentation_priors: AntsAtroposN4ShSegmentationPriors,
    max_n4_atropos_iterations: int | None = None,
    max_atropos_iterations: int | None = None,
    mrf: str | None = None,
    denoise_anatomical_images: typing.Literal[0, 1] | None = None,
    posterior_formulation: typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None = None,
    label_propagation: str | None = None,
    posterior_label_for_n4_weight_mask: str | None = None,
    image_file_suffix: str | None = None,
    keep_temporary_files: typing.Literal[0, 1] | None = None,
    use_random_seeding: typing.Literal[0, 1] | None = None,
    atropos_segmentation_prior_weight: float | None = None,
    n4_convergence: str | None = None,
    n4_shrink_factor: int | None = None,
    n4_bspline_params: str | None = None,
    atropos_segmentation_icm: str | None = None,
    atropos_segmentation_use_euclidean_distance: typing.Literal[0, 1] | None = None,
    test_debug_mode: int | None = None,
    runner: Runner | None = None,
) -> AntsAtroposN4ShOutputs:
    """
    antsAtroposN4.sh iterates between N4 <-> Atropos to improve segmentation
    results.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: 2 or 3, for 2- or 3-dimensional image.
        input_image: Anatomical image, typically T1. If more than one\
            anatomical image is specified, subsequent images are also used during\
            the segmentation process.
        mask_image: Binary mask defining the region of interest.
        number_of_classes: Number of classes defining the segmentation.
        output_prefix: The following images are created:\
            {output_prefix}N4Corrected.{output_suffix},\
            {output_prefix}Segmentation.{output_suffix},\
            {output_prefix}SegmentationPosteriors.{output_suffix}.
        segmentation_priors: Prior probability images initializing the\
            segmentation. Specified using c-style formatting, e.g. -p\
            labelsPriors%02d.nii.gz. If this is not specified, k-means\
            initialization is used instead.
        max_n4_atropos_iterations: Maximum number of (outer loop) iterations\
            between N4 <-> Atropos (default = 15).
        max_atropos_iterations: Maximum number of (inner loop) iterations in\
            Atropos (default = 3).
        mrf: Specifies MRF prior (of the form '[ weight,neighborhood ]', e.g.\
            '[ 0.1,1x1x1 ]' which is default).
        denoise_anatomical_images: Denoise anatomical images (1) or not (0)\
            (default = 1).
        posterior_formulation: Posterior formulation and whether or not to use\
            mixture model proportions. e.g 'Socrates[ 1 ]' (default) or 'Aristotle[\
            1 ]'. Choose the latter if you want to use the distance priors, see\
            also the -l option for label propagation control (default = 'Socrates[\
            1 ]').
        label_propagation: Incorporate a distance prior into the 'Aristotle'\
            posterior formulation. Should be of the form 'label[\
            lambda,boundaryProbability ]' where label is a value of 1,2,3,...\
            denoting label ID. The label probability for anything outside the\
            current label\
            \
            = boundaryProbability * exp( -lambda * distanceFromBoundary )\
            \
            Intuitively, smaller lambda values will increase the spatial\
            capture range of the distance prior. To apply to all label values,\
            simply omit specifying the label, i.e. -l '[\
            lambda,boundaryProbability ]'.
        posterior_label_for_n4_weight_mask: Which posterior probability image\
            should be used to define the N4 weight mask. Can also specify multiple\
            posteriors in which case the chosen posteriors are combined.
        image_file_suffix: Any of the standard ITK IO formats e.g. nrrd, nii.gz\
            (default), mhd.
        keep_temporary_files: Keep temporary files on disk (1) or delete them\
            (0) (default = 0).
        use_random_seeding: Use random number generated from system clock in\
            Atropos (default = 1).
        atropos_segmentation_prior_weight: Atropos spatial prior probability\
            weight for the segmentation (default = 0.25).
        n4_convergence: Convergence parameters for N4, see '-c' option in\
            N4BiasFieldCorrection (default = [50x50x50x50,0.0000001]).
        n4_shrink_factor: Shrink factor for N4 (default = 4).
        n4_bspline_params: N4 b-spline specification, see '-b' option in\
            N4BiasFieldCorrection (default = [200,0,0,0]).
        atropos_segmentation_icm: ICM parameters for segmentation, see '-g'\
            option in Atropos (default = [1,1]).
        atropos_segmentation_use_euclidean_distance: Use euclidean distances in\
            distance prior formulation (1) or not (0), see Atropos usage for\
            details (default = 1).
        test_debug_mode: If > 0, attempts to continue after errors.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAtroposN4ShOutputs`).
    """
    if max_n4_atropos_iterations is not None and not (1 <= max_n4_atropos_iterations): 
        raise ValueError(f"'max_n4_atropos_iterations' must be greater than 1 <= x but was {max_n4_atropos_iterations}")
    if max_atropos_iterations is not None and not (1 <= max_atropos_iterations): 
        raise ValueError(f"'max_atropos_iterations' must be greater than 1 <= x but was {max_atropos_iterations}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_ATROPOS_N4_SH_METADATA)
    cargs = []
    cargs.append("antsAtroposN4.sh")
    cargs.extend([
        "-d",
        str(image_dimension)
    ])
    cargs.extend([
        "-a",
        execution.input_file(input_image)
    ])
    cargs.extend([
        "-x",
        execution.input_file(mask_image)
    ])
    cargs.extend([
        "-c",
        str(number_of_classes)
    ])
    cargs.extend([
        "-o",
        output_prefix
    ])
    if max_n4_atropos_iterations is not None:
        cargs.extend([
            "-m",
            str(max_n4_atropos_iterations)
        ])
    if max_atropos_iterations is not None:
        cargs.extend([
            "-n",
            str(max_atropos_iterations)
        ])
    cargs.extend([
        "-p",
        *segmentation_priors.run(execution)
    ])
    if mrf is not None:
        cargs.extend([
            "-r",
            mrf
        ])
    if denoise_anatomical_images is not None:
        cargs.extend([
            "-g",
            str(denoise_anatomical_images)
        ])
    if posterior_formulation is not None:
        cargs.extend([
            "-b",
            posterior_formulation
        ])
    if label_propagation is not None:
        cargs.extend([
            "-l",
            label_propagation
        ])
    if posterior_label_for_n4_weight_mask is not None:
        cargs.extend([
            "-y",
            posterior_label_for_n4_weight_mask
        ])
    if image_file_suffix is not None:
        cargs.extend([
            "-s",
            image_file_suffix
        ])
    if keep_temporary_files is not None:
        cargs.extend([
            "-k",
            str(keep_temporary_files)
        ])
    if use_random_seeding is not None:
        cargs.extend([
            "-u",
            str(use_random_seeding)
        ])
    if atropos_segmentation_prior_weight is not None:
        cargs.extend([
            "-w",
            str(atropos_segmentation_prior_weight)
        ])
    if n4_convergence is not None:
        cargs.extend([
            "-e",
            n4_convergence
        ])
    if n4_shrink_factor is not None:
        cargs.extend([
            "-f",
            str(n4_shrink_factor)
        ])
    if n4_bspline_params is not None:
        cargs.extend([
            "-q",
            n4_bspline_params
        ])
    if atropos_segmentation_icm is not None:
        cargs.extend([
            "-i",
            atropos_segmentation_icm
        ])
    if atropos_segmentation_use_euclidean_distance is not None:
        cargs.extend([
            "-j",
            str(atropos_segmentation_use_euclidean_distance)
        ])
    if test_debug_mode is not None:
        cargs.extend([
            "-z",
            str(test_debug_mode)
        ])
    ret = AntsAtroposN4ShOutputs(
        root=execution.output_file("."),
        n4_corrected=execution.output_file(output_prefix + "N4Corrected.[OUTPUT_SUFFIX]"),
        segmentation=execution.output_file(output_prefix + "Segmentation.[OUTPUT_SUFFIX]"),
        segmentation_posteriors=execution.output_file(output_prefix + "SegmentationPosteriors.[OUTPUT_SUFFIX]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_ATROPOS_N4_SH_METADATA",
    "AntsAtroposN4ShOutputs",
    "AntsAtroposN4ShSegmentationPriors",
    "ants_atropos_n4_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

GREEDY_METADATA = Metadata(
    id="81e25c6c4fb84453185826b4258b91dafaf56f30.boutiques",
    name="greedy",
    package="greedy",
    container_image_tag="pyushkevich/itksnap:v3.8.2",
)


@dataclasses.dataclass
class GreedyInputImages:
    """
    Image pair (may be repeated).
    """
    fixed: InputPathType
    """The fixed image"""
    moving: InputPathType
    """The moving image"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.fixed))
        cargs.append(execution.input_file(self.moving))
        return cargs


class GreedyInvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyInvert | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    inverted_warp: OutputPathType
    """The inverted warp file"""


@dataclasses.dataclass
class GreedyInvert:
    """
    Invert previously computed warp.
    """
    inwarp: InputPathType
    """The input warp to be inverted"""
    outwarp: str
    """The filename for the inverted warp output"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.inwarp))
        cargs.append(self.outwarp)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyInvertOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyInvertOutputs`).
        """
        ret = GreedyInvertOutputs(
            root=execution.output_file("."),
            inverted_warp=execution.output_file(self.outwarp),
        )
        return ret


class GreedyRootOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyRoot | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    root_warp: OutputPathType
    """The 2^N-th root of the input warp"""


@dataclasses.dataclass
class GreedyRoot:
    """
    Convert 2^N-th root of a warp.
    """
    inwarp: InputPathType
    """The input warp"""
    outwarp: str
    """The filename for the root warp output"""
    n: int
    """The value of N for 2^N-th root"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.inwarp))
        cargs.append(self.outwarp)
        cargs.append(str(self.n))
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyRootOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyRootOutputs`).
        """
        ret = GreedyRootOutputs(
            root=execution.output_file("."),
            root_warp=execution.output_file(self.outwarp),
        )
        return ret


class GreedyJacobianOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyJacobian | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    jacobian_determinant: OutputPathType
    """The Jacobian determinant of the warp"""


@dataclasses.dataclass
class GreedyJacobian:
    """
    Compute the Jacobian determinant of the warp.
    """
    inwarp: InputPathType
    """The input warp"""
    outjac: str
    """The filename for the Jacobian determinant output"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.inwarp))
        cargs.append(self.outjac)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyJacobianOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyJacobianOutputs`).
        """
        ret = GreedyJacobianOutputs(
            root=execution.output_file("."),
            jacobian_determinant=execution.output_file(self.outjac),
        )
        return ret


@dataclasses.dataclass
class GreedyMetric:
    """
    Metric for the entire registration.
    """
    metric_type: typing.Literal["SSD", "MI", "NMI", "NCC", "MAHAL"]
    """Type of metric to use"""
    metric_param: float | None = None
    """Parameter for the metric (e.g., radius for NCC)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(self.metric_type)
        if self.metric_param is not None:
            cargs.append(str(self.metric_param))
        return cargs


@dataclasses.dataclass
class GreedyTjr:
    """
    Apply a regularization penalty based on the Jacobian of a tetrahedral mesh.
    """
    mesh: InputPathType
    """The tetrahedral mesh file"""
    weight: float
    """The weight of the penalty term"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.mesh))
        cargs.append(str(self.weight))
        return cargs


@dataclasses.dataclass
class GreedySearch:
    """
    Random search over rigid transforms before starting optimization.
    """
    n: int
    """Number of iterations for random search"""
    rot: str
    """Standard deviation of random rotation angle (degrees) or 'any' or
    'flip'"""
    tran: float
    """Standard deviation of random offset in physical units"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(str(self.n))
        cargs.append(self.rot)
        cargs.append(str(self.tran))
        return cargs


class GreedyResliceMovingImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyResliceMovingImage | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    resliced_image: OutputPathType
    """The resliced output image"""


@dataclasses.dataclass
class GreedyResliceMovingImage:
    """
    Moving/output image pair for reslicing.
    """
    moving: InputPathType
    """The moving image to be resliced"""
    output: str
    """The filename for the resliced output image"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.moving))
        cargs.append(self.output)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyResliceMovingImageOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyResliceMovingImageOutputs`).
        """
        ret = GreedyResliceMovingImageOutputs(
            root=execution.output_file("."),
            resliced_image=execution.output_file(self.output),
        )
        return ret


class GreedyResliceSurfaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyResliceSurface | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    warped_mesh: OutputPathType
    """The warped output mesh"""


@dataclasses.dataclass
class GreedyResliceSurface:
    """
    Fixed/output surface pair (vertices are warped from fixed space to moving).
    """
    inmesh: InputPathType
    """The input mesh file"""
    outmesh: str
    """The filename for the output warped mesh"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.inmesh))
        cargs.append(self.outmesh)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyResliceSurfaceOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyResliceSurfaceOutputs`).
        """
        ret = GreedyResliceSurfaceOutputs(
            root=execution.output_file("."),
            warped_mesh=execution.output_file(self.outmesh),
        )
        return ret


class GreedyResliceSimplexJacobianOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GreedyResliceSimplexJacobian | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    jacobian_mesh: OutputPathType
    """The output mesh with computed Jacobian determinants"""


@dataclasses.dataclass
class GreedyResliceSimplexJacobian:
    """
    Compute Jacobian determinant for a simplex mesh in fixed space.
    """
    inmesh: InputPathType
    """The input simplex mesh file"""
    outmesh: str
    """The filename for the output mesh with Jacobian determinants"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.inmesh))
        cargs.append(self.outmesh)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> GreedyResliceSimplexJacobianOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `GreedyResliceSimplexJacobianOutputs`).
        """
        ret = GreedyResliceSimplexJacobianOutputs(
            root=execution.output_file("."),
            jacobian_mesh=execution.output_file(self.outmesh),
        )
        return ret


class GreedyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `greedy_(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType | None
    """Output file from affine or deformable registration"""
    invert: GreedyInvertOutputs | None
    """Outputs from `GreedyInvert`."""
    root_: GreedyRootOutputs | None
    """Outputs from `GreedyRoot`."""
    jacobian: GreedyJacobianOutputs | None
    """Outputs from `GreedyJacobian`."""
    reslice_moving_image: GreedyResliceMovingImageOutputs | None
    """Outputs from `GreedyResliceMovingImage`."""
    reslice_surface: GreedyResliceSurfaceOutputs | None
    """Outputs from `GreedyResliceSurface`."""
    reslice_simplex_jacobian: GreedyResliceSimplexJacobianOutputs | None
    """Outputs from `GreedyResliceSimplexJacobian`."""


def greedy_(
    dimensions: int,
    input_images: GreedyInputImages | None = None,
    output: str | None = None,
    affine: bool = False,
    brute: str | None = None,
    moments: typing.Literal[1, 2] | None = None,
    reslice: list[InputPathType] | None = None,
    invert: GreedyInvert | None = None,
    root: GreedyRoot | None = None,
    jacobian: GreedyJacobian | None = None,
    metric_mode: bool = False,
    defopt: bool = False,
    weight: float | None = None,
    metric: GreedyMetric | None = None,
    step_size: str | None = None,
    iterations: str | None = None,
    threads: int | None = None,
    fixed_mask: InputPathType | None = None,
    gm_trim: float | None = None,
    moving_mask: InputPathType | None = None,
    wncc_mask_dilate: bool = False,
    reference_image: InputPathType | None = None,
    ref_pad: float | None = None,
    background: str | None = None,
    input_transform: InputPathType | None = None,
    zero_last_dimension: bool = False,
    time_step_mode: typing.Literal["CONST", "SCALE", "SCALEDOWN"] | None = None,
    smoothing: str | None = None,
    inverse_warp: InputPathType | None = None,
    root_warp: InputPathType | None = None,
    warp_precision: float | None = None,
    noise: float | None = None,
    exponent: float | None = None,
    svf_mode: bool = False,
    svlb: bool = False,
    sv_incompr: bool = False,
    initial_warp: InputPathType | None = None,
    tjr: GreedyTjr | None = None,
    wr: float | None = None,
    initial_affine: InputPathType | None = None,
    ia_identity: bool = False,
    ia_voxel_grid: bool = False,
    ia_image_centers: bool = False,
    ia_image_side: str | None = None,
    ia_moments: typing.Literal[1, 2] | None = None,
    affine_dof: typing.Literal[6, 7, 12] | None = None,
    jitter: float | None = None,
    search: GreedySearch | None = None,
    det: typing.Literal[-1, 1] | None = None,
    cov_id: bool = False,
    fixed_reslicing_image: InputPathType | None = None,
    reslice_moving_image: GreedyResliceMovingImage | None = None,
    reslice_surface: GreedyResliceSurface | None = None,
    interpolation: typing.Literal["NN", "LINEAR", "LABEL"] | None = None,
    reslice_background: float | None = None,
    reslice_datatype: typing.Literal["auto", "double", "float", "uint", "int", "ushort", "short", "uchar", "char"] | None = None,
    reslice_composite: InputPathType | None = None,
    reslice_jacobian: InputPathType | None = None,
    reslice_simplex_jacobian: GreedyResliceSimplexJacobian | None = None,
    reslice_mask: InputPathType | None = None,
    metric_gradient: InputPathType | None = None,
    debug_deriv: bool = False,
    debug_deriv_eps: float | None = None,
    debug_aff_obj: bool = False,
    dump_pyramid: bool = False,
    dump_moving: bool = False,
    dump_frequency: float | None = None,
    dump_prefix: str | None = None,
    powell: bool = False,
    float_: bool = False,
    version: bool = False,
    verbosity: typing.Literal[0, 1, 2] | None = None,
    runner: Runner | None = None,
) -> GreedyOutputs:
    """
    Paul's greedy diffeomorphic registration implementation.
    
    Author: Greedy Developers
    
    URL: https://sites.google.com/view/greedyreg/about
    
    Args:
        dimensions: Number of image dimensions.
        input_images: Image pair (may be repeated).
        output: Output file (matrix in affine mode; image in deformable mode,\
            metric computation mode; ignored in reslicing mode).
        affine: Perform affine registration and save to output (-o).
        brute: Perform a brute force search around each voxel.
        moments: Perform moments of inertia rigid alignment of given order (1\
            or 2).
        reslice: Reslice images instead of doing registration.
        invert: Invert previously computed warp.
        root: Convert 2^N-th root of a warp.
        jacobian: Compute the Jacobian determinant of the warp.
        metric_mode: Compute metric between images.
        defopt: Deformable optimization mode (experimental).
        weight: Weight of the next -i pair.
        metric: Metric for the entire registration.
        step_size: Step size for optimization (default = 1.0), may also be\
            specified per level (e.g. 0.3x0.1).
        iterations: Number of iterations per level of multi-res.
        threads: Number of allowed concurrent threads.
        fixed_mask: Fixed image mask (metric gradients computed only over the\
            mask).
        gm_trim: Generate the fixed image mask by trimming the extent of the\
            fixed image by given radius.
        moving_mask: Moving image mask (pixels outside are excluded from metric\
            computation).
        wncc_mask_dilate: Specifies that fixed and moving masks should be\
            dilated by the radius of the WNCC metric during registration.
        reference_image: Use supplied image, rather than fixed image to define\
            the reference space.
        ref_pad: Define the reference space by padding the fixed image by\
            radius.
        background: Value to fill missing values when mapping fixed and moving\
            images to reference space.
        input_transform: Specify transforms (matrices, warps) that map moving\
            image to reference space.
        zero_last_dimension: Sets sigmas and NCC radius to zero in the last\
            dimension for 2D/3D registration.
        time_step_mode: Time step behavior mode.
        smoothing: Smoothing for the greedy update step (e.g., '1.732vox\
            0.7071vox').
        inverse_warp: Compute and write the inverse of the warp field.
        root_warp: Compute and write the (2^N-th) root of the warp field.
        warp_precision: Saved warp precision in voxels (default=0.1; 0 for no\
            compression).
        noise: Standard deviation of white noise added to moving/fixed images\
            when using NCC metric.
        exponent: The exponent used for warp inversion, root computation, and\
            in stationary velocity field mode.
        svf_mode: Perform registration using the stationary velocity model.
        svlb: Use more accurate but expensive update of v in stationary\
            velocity field mode.
        sv_incompr: Incompressibility mode, implements Mansi et al. 2011\
            iLogDemons.
        initial_warp: Specifies the initial warp to start iteration from.
        tjr: Apply a regularization penalty based on the Jacobian of a\
            tetrahedral mesh.
        wr: Weight of SVF smoothness regularization term (default: 1000).
        initial_affine: Initial affine matrix for optimization.
        ia_identity: Initialize affine matrix based on NIFTI headers.
        ia_voxel_grid: Initialize affine matrix so that voxels with\
            corresponding indices align.
        ia_image_centers: Initialize affine matrix based on matching image\
            centers.
        ia_image_side: Initialize affine matrix based on matching center of one\
            image side.
        ia_moments: Initialize affine matrix based on matching moments of\
            inertia.
        affine_dof: Degrees of freedom for affine registration. 6=rigid,\
            7=similarity, 12=affine.
        jitter: Jitter (in voxel units) applied to sample points (default: 0.5).
        search: Random search over rigid transforms before starting\
            optimization.
        det: Force the determinant of transform to be either 1 (no flip) or -1\
            (flip).
        cov_id: Assume identity covariance (match centers and do flips only, no\
            rotation).
        fixed_reslicing_image: Fixed image for reslicing.
        reslice_moving_image: Moving/output image pair for reslicing.
        reslice_surface: Fixed/output surface pair (vertices are warped from\
            fixed space to moving).
        interpolation: Interpolation for the next pair (NN, LINEAR*, LABEL\
            sigma).
        reslice_background: Background (i.e. outside) intensity for the next\
            pair (default 0).
        reslice_datatype: Data type for the next pair.
        reslice_composite: Write composed transforms to outwarp.
        reslice_jacobian: Write Jacobian determinant image to outjacobian.
        reslice_simplex_jacobian: Compute Jacobian determinant for a simplex\
            mesh in fixed space.
        reslice_mask: A binary mask for the fixed image; zero values will be\
            overwritten with background.
        metric_gradient: Write the gradient of the metric to file.
        debug_deriv: Enable periodic checks of derivatives (debug).
        debug_deriv_eps: Epsilon for derivative debugging.
        debug_aff_obj: Plot affine objective in neighborhood of -ia matrix.
        dump_pyramid: Dump the image pyramid at the start of the registration.
        dump_moving: Dump moving image at each iter.
        dump_frequency: Dump frequency.
        dump_prefix: Prefix for dump files (may be a path).
        powell: Use Powell's method instead of LGBFS.
        float_: Use single precision floating point (off by default).
        version: Print version info.
        verbosity: Verbosity level (0: none, 1: default, 2: verbose).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GreedyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(GREEDY_METADATA)
    cargs = []
    cargs.append("greedy")
    cargs.extend([
        "-d",
        str(dimensions)
    ])
    if input_images is not None:
        cargs.extend([
            "-i",
            *input_images.run(execution)
        ])
    if output is not None:
        cargs.extend([
            "-o",
            output
        ])
    if affine:
        cargs.append("-a")
    if brute is not None:
        cargs.extend([
            "-brute",
            brute
        ])
    if moments is not None:
        cargs.extend([
            "-moments",
            str(moments)
        ])
    if reslice is not None:
        cargs.extend([
            "-r",
            *[execution.input_file(f) for f in reslice]
        ])
    if invert is not None:
        cargs.extend([
            "-iw",
            *invert.run(execution)
        ])
    if root is not None:
        cargs.extend([
            "-root",
            *root.run(execution)
        ])
    if jacobian is not None:
        cargs.extend([
            "-jac",
            *jacobian.run(execution)
        ])
    if metric_mode:
        cargs.append("-metric")
    if defopt:
        cargs.append("-defopt")
    if weight is not None:
        cargs.extend([
            "-w",
            str(weight)
        ])
    if metric is not None:
        cargs.extend([
            "-m",
            *metric.run(execution)
        ])
    if step_size is not None:
        cargs.extend([
            "-e",
            step_size
        ])
    if iterations is not None:
        cargs.extend([
            "-n",
            iterations
        ])
    if threads is not None:
        cargs.extend([
            "-threads",
            str(threads)
        ])
    if fixed_mask is not None:
        cargs.extend([
            "-gm",
            execution.input_file(fixed_mask)
        ])
    if gm_trim is not None:
        cargs.extend([
            "-gm-trim",
            str(gm_trim)
        ])
    if moving_mask is not None:
        cargs.extend([
            "-mm",
            execution.input_file(moving_mask)
        ])
    if wncc_mask_dilate:
        cargs.append("-wncc-mask-dilate")
    if reference_image is not None:
        cargs.extend([
            "-ref",
            execution.input_file(reference_image)
        ])
    if ref_pad is not None:
        cargs.extend([
            "-ref-pad",
            str(ref_pad)
        ])
    if background is not None:
        cargs.extend([
            "-bg",
            background
        ])
    if input_transform is not None:
        cargs.extend([
            "-it",
            execution.input_file(input_transform)
        ])
    if zero_last_dimension:
        cargs.append("-z")
    if time_step_mode is not None:
        cargs.extend([
            "-tscale",
            time_step_mode
        ])
    if smoothing is not None:
        cargs.extend([
            "-s",
            smoothing
        ])
    if inverse_warp is not None:
        cargs.extend([
            "-oinv",
            execution.input_file(inverse_warp)
        ])
    if root_warp is not None:
        cargs.extend([
            "-oroot",
            execution.input_file(root_warp)
        ])
    if warp_precision is not None:
        cargs.extend([
            "-wp",
            str(warp_precision)
        ])
    if noise is not None:
        cargs.extend([
            "-noise",
            str(noise)
        ])
    if exponent is not None:
        cargs.extend([
            "-exp",
            str(exponent)
        ])
    if svf_mode:
        cargs.append("-sv")
    if svlb:
        cargs.append("-svlb")
    if sv_incompr:
        cargs.append("-sv-incompr")
    if initial_warp is not None:
        cargs.extend([
            "-id",
            execution.input_file(initial_warp)
        ])
    if tjr is not None:
        cargs.extend([
            "-tjr",
            *tjr.run(execution)
        ])
    if wr is not None:
        cargs.extend([
            "-wr",
            str(wr)
        ])
    if initial_affine is not None:
        cargs.extend([
            "-ia",
            execution.input_file(initial_affine)
        ])
    if ia_identity:
        cargs.append("-ia-identity")
    if ia_voxel_grid:
        cargs.append("-ia-voxel-grid")
    if ia_image_centers:
        cargs.append("-ia-image-centers")
    if ia_image_side is not None:
        cargs.extend([
            "-ia-image-side",
            ia_image_side
        ])
    if ia_moments is not None:
        cargs.extend([
            "-ia-moments",
            str(ia_moments)
        ])
    if affine_dof is not None:
        cargs.extend([
            "-dof",
            str(affine_dof)
        ])
    if jitter is not None:
        cargs.extend([
            "-jitter",
            str(jitter)
        ])
    if search is not None:
        cargs.extend([
            "-search",
            *search.run(execution)
        ])
    if det is not None:
        cargs.extend([
            "-det",
            str(det)
        ])
    if cov_id:
        cargs.append("-cov-id")
    if fixed_reslicing_image is not None:
        cargs.extend([
            "-rf",
            execution.input_file(fixed_reslicing_image)
        ])
    if reslice_moving_image is not None:
        cargs.extend([
            "-rm",
            *reslice_moving_image.run(execution)
        ])
    if reslice_surface is not None:
        cargs.extend([
            "-rs",
            *reslice_surface.run(execution)
        ])
    if interpolation is not None:
        cargs.extend([
            "-ri",
            interpolation
        ])
    if reslice_background is not None:
        cargs.extend([
            "-rb",
            str(reslice_background)
        ])
    if reslice_datatype is not None:
        cargs.extend([
            "-rt",
            reslice_datatype
        ])
    if reslice_composite is not None:
        cargs.extend([
            "-rc",
            execution.input_file(reslice_composite)
        ])
    if reslice_jacobian is not None:
        cargs.extend([
            "-rj",
            execution.input_file(reslice_jacobian)
        ])
    if reslice_simplex_jacobian is not None:
        cargs.extend([
            "-rsj",
            *reslice_simplex_jacobian.run(execution)
        ])
    if reslice_mask is not None:
        cargs.extend([
            "-rk",
            execution.input_file(reslice_mask)
        ])
    if metric_gradient is not None:
        cargs.extend([
            "-og",
            execution.input_file(metric_gradient)
        ])
    if debug_deriv:
        cargs.append("-debug-deriv")
    if debug_deriv_eps is not None:
        cargs.extend([
            "-debug-deriv-eps",
            str(debug_deriv_eps)
        ])
    if debug_aff_obj:
        cargs.append("-debug-aff-obj")
    if dump_pyramid:
        cargs.append("-dump-pyramid")
    if dump_moving:
        cargs.append("-dump-moving")
    if dump_frequency is not None:
        cargs.extend([
            "-dump-freq",
            str(dump_frequency)
        ])
    if dump_prefix is not None:
        cargs.extend([
            "-dump-prefix",
            dump_prefix
        ])
    if powell:
        cargs.append("-powell")
    if float_:
        cargs.append("-float")
    if version:
        cargs.append("-version")
    if verbosity is not None:
        cargs.extend([
            "-V",
            str(verbosity)
        ])
    ret = GreedyOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output) if (output is not None) else None,
        invert=invert.outputs(execution) if invert else None,
        root_=root.outputs(execution) if root else None,
        jacobian=jacobian.outputs(execution) if jacobian else None,
        reslice_moving_image=reslice_moving_image.outputs(execution) if reslice_moving_image else None,
        reslice_surface=reslice_surface.outputs(execution) if reslice_surface else None,
        reslice_simplex_jacobian=reslice_simplex_jacobian.outputs(execution) if reslice_simplex_jacobian else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "GREEDY_METADATA",
    "GreedyInputImages",
    "GreedyInvert",
    "GreedyInvertOutputs",
    "GreedyJacobian",
    "GreedyJacobianOutputs",
    "GreedyMetric",
    "GreedyOutputs",
    "GreedyResliceMovingImage",
    "GreedyResliceMovingImageOutputs",
    "GreedyResliceSimplexJacobian",
    "GreedyResliceSimplexJacobianOutputs",
    "GreedyResliceSurface",
    "GreedyResliceSurfaceOutputs",
    "GreedyRoot",
    "GreedyRootOutputs",
    "GreedySearch",
    "GreedyTjr",
    "greedy_",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CONVERT_SURFACE_METADATA = Metadata(
    id="2b7452b8bf057bc43ac7efc4c5a01abd5236749c.boutiques",
    name="ConvertSurface",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ConvertSurfaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `convert_surface(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_surface_file: OutputPathType
    """The converted output surface file."""


def convert_surface(
    input_surface: str,
    output_surface: str,
    surface_volume: str | None = None,
    transform_tlrc: bool = False,
    mni_lpi: bool = False,
    ixmat_1_d: str | None = None,
    native: bool = False,
    runner: Runner | None = None,
) -> ConvertSurfaceOutputs:
    """
    Reads in a surface and writes it out in another format. Only fields pertinent to
    SUMA are preserved.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_surface: Specifies the input surface.
        output_surface: Specifies the output surface.
        surface_volume: Specifies a surface volume.
        transform_tlrc: Apply Talairach transform.
        mni_lpi: Turn AFNI tlrc coordinates (RAI) into MNI coord space in LPI.
        ixmat_1_d: Apply inverse transformation specified in 1D file.
        native: Write the output surface in the coordinate system native to its\
            format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertSurfaceOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_SURFACE_METADATA)
    cargs = []
    cargs.append("ConvertSurface")
    cargs.extend([
        "-i",
        input_surface
    ])
    cargs.extend([
        "-o",
        output_surface
    ])
    if surface_volume is not None:
        cargs.extend([
            "-sv",
            surface_volume
        ])
    if transform_tlrc:
        cargs.append("-tlrc")
    if mni_lpi:
        cargs.append("-MNI_lpi")
    if ixmat_1_d is not None:
        cargs.extend([
            "-ixmat_1D",
            ixmat_1_d
        ])
    if native:
        cargs.append("-native")
    ret = ConvertSurfaceOutputs(
        root=execution.output_file("."),
        output_surface_file=execution.output_file(output_surface),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CONVERT_SURFACE_METADATA",
    "ConvertSurfaceOutputs",
    "convert_surface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TKMEDITFV_METADATA = Metadata(
    id="c54cb02c761041c3c2bd1fa2a8964899175b9969.boutiques",
    name="tkmeditfv",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class TkmeditfvOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tkmeditfv(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tkmeditfv(
    mainvol: InputPathType,
    subject: str | None = None,
    aux_volume: InputPathType | None = None,
    seg_volume: InputPathType | None = None,
    overlay: InputPathType | None = None,
    timecourse: InputPathType | None = None,
    overlay_registration: InputPathType | None = None,
    surface: list[str] | None = None,
    extra_volumes: list[InputPathType] | None = None,
    crs_location: list[float] | None = None,
    zoom_level: float | None = None,
    additional_segments: list[InputPathType] | None = None,
    load_white: bool = False,
    load_pial: bool = False,
    load_orig: bool = False,
    load_orig_nofix: bool = False,
    load_smoothwm_nofix: bool = False,
    load_white_preaparc: bool = False,
    load_inflated: bool = False,
    annot: str | None = None,
    load_aparc: bool = False,
    surfext: str | None = None,
    seg_outline: bool = False,
    intensity_minmax: list[float] | None = None,
    load_defects: bool = False,
    load_defect_pointset: bool = False,
    trilin_interpolation: bool = False,
    neurological_orientation: bool = False,
    rotate_around_cursor: bool = False,
    vgl_display: bool = False,
    use_tkmedit: bool = False,
    load_aparc_aseg: bool = False,
    runner: Runner | None = None,
) -> TkmeditfvOutputs:
    """
    A wrapper script to run Freeview with arguments similar to tkmedit, providing
    both functionalities.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mainvol: Main volume file path.
        subject: Subject's name.
        aux_volume: Auxiliary volume file path.
        seg_volume: Segmentation volume file path.
        overlay: Overlay volume file path.
        timecourse: Timecourse for overlay.
        overlay_registration: Overlay timecourse registration file.
        surface: Load surface with optional color.
        extra_volumes: Load extra volume(s).
        crs_location: Place cursor at given (col, row, slice) location.
        zoom_level: Set zoom level.
        additional_segments: Add additional segmentations.
        load_white: Load lh and rh.white surfaces.
        load_pial: Load lh and rh.pial surfaces.
        load_orig: Load lh and rh.orig surfaces.
        load_orig_nofix: Load lh and rh.orig.nofix surfaces.
        load_smoothwm_nofix: Load lh and rh.smoothwm.nofix surfaces.
        load_white_preaparc: Load lh and rh.white.preaparc surfaces.
        load_inflated: Load lh and rh.inflated surfaces.
        annot: Load annotation file.
        load_aparc: Load aparc annotation.
        surfext: Add extension to the name of the surface.
        seg_outline: Enable segmentation outline mode.
        intensity_minmax: Set intensity min and max on first volume.
        load_defects: Load info needed to evaluate defects.
        load_defect_pointset: Load defect pointset.
        trilin_interpolation: Use trilinear interpolation for volume overlays.
        neurological_orientation: Use neurological orientation instead of\
            radiological.
        rotate_around_cursor: Rotate around cursor in 3D view.
        vgl_display: Set VGL_DISPLAY and run with vglrun.
        use_tkmedit: Use tkmedit instead of freeview.
        load_aparc_aseg: Load aparc+aseg.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TkmeditfvOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TKMEDITFV_METADATA)
    cargs = []
    cargs.append("tkmeditfv")
    if subject is not None:
        cargs.append(subject)
    cargs.append(execution.input_file(mainvol))
    if aux_volume is not None:
        cargs.extend([
            "-aux",
            execution.input_file(aux_volume)
        ])
    if seg_volume is not None:
        cargs.extend([
            "-seg",
            execution.input_file(seg_volume)
        ])
    if overlay is not None:
        cargs.extend([
            "-ov",
            execution.input_file(overlay)
        ])
    if timecourse is not None:
        cargs.extend([
            "-t",
            execution.input_file(timecourse)
        ])
    if overlay_registration is not None:
        cargs.extend([
            "-reg",
            execution.input_file(overlay_registration)
        ])
    if surface is not None:
        cargs.extend([
            "-surf",
            *surface
        ])
    if extra_volumes is not None:
        cargs.extend([
            "-vol",
            *[execution.input_file(f) for f in extra_volumes]
        ])
    if crs_location is not None:
        cargs.extend([
            "-crs",
            *map(str, crs_location)
        ])
    if zoom_level is not None:
        cargs.extend([
            "-zoom",
            str(zoom_level)
        ])
    if additional_segments is not None:
        cargs.extend([
            "-seg2",
            *[execution.input_file(f) for f in additional_segments]
        ])
    if load_white:
        cargs.append("-white")
    if load_pial:
        cargs.append("-pial")
    if load_orig:
        cargs.append("-orig")
    if load_orig_nofix:
        cargs.append("-orig.nofix")
    if load_smoothwm_nofix:
        cargs.append("-smoothwm.nofix")
    if load_white_preaparc:
        cargs.append("-white.preaparc")
    if load_inflated:
        cargs.append("-inflated")
    if annot is not None:
        cargs.extend([
            "-annot",
            annot
        ])
    if load_aparc:
        cargs.append("-aparc")
    if surfext is not None:
        cargs.extend([
            "-surfext",
            surfext
        ])
    if seg_outline:
        cargs.append("-seg-outline")
    if intensity_minmax is not None:
        cargs.extend([
            "-main-minmax",
            *map(str, intensity_minmax)
        ])
    if load_defects:
        cargs.append("-defects")
    if load_defect_pointset:
        cargs.append("-defectps")
    if trilin_interpolation:
        cargs.append("-trilin")
    if neurological_orientation:
        cargs.append("-neuro")
    if rotate_around_cursor:
        cargs.append("-rotate-around-cursor")
    if vgl_display:
        cargs.append("-vgl")
    if use_tkmedit:
        cargs.append("-tkmedit")
    if load_aparc_aseg:
        cargs.append("-aparc+aseg")
    ret = TkmeditfvOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TKMEDITFV_METADATA",
    "TkmeditfvOutputs",
    "tkmeditfv",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_RANK_METADATA = Metadata(
    id="05c7817af68d191ce35831e8d5e9772041297b36.boutiques",
    name="3dRank",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dRankOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_rank(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset_head: OutputPathType | None
    """Main output dataset in AFNI format (.HEAD file)"""
    output_dataset_brik: OutputPathType | None
    """Main output dataset in AFNI format (.BRIK file)"""
    rank_map_file: OutputPathType | None
    """Rank map 1D file showing the mapping from the rank to the integral values
    in the input dataset"""


def v_3d_rank(
    input_datasets: list[InputPathType],
    output_prefix: str | None = None,
    version_info: bool = False,
    help_info: bool = False,
    runner: Runner | None = None,
) -> V3dRankOutputs:
    """
    Replaces voxel values by their rank in the set of values collected over all
    voxels in all input datasets.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_datasets: Input datasets. Acceptable data types are: byte, short,\
            and floats.
        output_prefix: Output prefix. If you have multiple datasets on input,\
            the prefix is preceded by r00., r01., etc. If no prefix is given, the\
            default is rank.DATASET1, rank.DATASET2, etc.
        version_info: Print author and version info.
        help_info: Print this help screen.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dRankOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_RANK_METADATA)
    cargs = []
    cargs.append("3dRank")
    cargs.extend([execution.input_file(f) for f in input_datasets])
    if output_prefix is not None:
        cargs.extend([
            "-prefix",
            output_prefix
        ])
    if version_info:
        cargs.append("-ver")
    if help_info:
        cargs.append("-help")
    ret = V3dRankOutputs(
        root=execution.output_file("."),
        output_dataset_head=execution.output_file(output_prefix + "*.HEAD") if (output_prefix is not None) else None,
        output_dataset_brik=execution.output_file(output_prefix + "*.BRIK") if (output_prefix is not None) else None,
        rank_map_file=execution.output_file(output_prefix + "*.1D") if (output_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dRankOutputs",
    "V_3D_RANK_METADATA",
    "v_3d_rank",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CVS_REGISTER_METADATA = Metadata(
    id="bb0813a0151b53cc7fd810cb07043b52e37cd609.boutiques",
    name="mri_cvs_register",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCvsRegisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_cvs_register(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    final_cvs_morph: OutputPathType | None
    """The final morph that combines correspondences recovered in all
    registration steps."""
    final_cvs_morphed_norm: OutputPathType | None
    """The CVS morphed norm.mgz file; final result of CVS with all registration
    steps contributions."""
    final_cvs_morphed_aseg: OutputPathType | None
    """The CVS morphed aseg.mgz file with all registration steps
    contributions."""
    step1_morphed_aseg: OutputPathType | None
    """Morphed aseg.mgz file result of CVS up to step (3)(i) and contains elatic
    morph and aseg-based nonlinear registration."""
    step1_morphed_norm: OutputPathType | None
    """Morphed norm.mgz file result of CVS up to step (3)(i) and contains elatic
    morph and aseg-based nonlinear registration."""


def mri_cvs_register(
    mov_subjid: str,
    template_subjid: str | None = None,
    templatedir: str | None = None,
    mni_flag: bool = False,
    outdir: str | None = None,
    asegfname: str | None = None,
    voltype: str | None = None,
    step1_flag: bool = False,
    step2_flag: bool = False,
    step3_flag: bool = False,
    noaseg_flag: bool = False,
    nointensity_flag: bool = False,
    hemi_flag: bool = False,
    masktargethemi_flag: bool = False,
    maskmovinghemi_flag: bool = False,
    nocleanup_flag: bool = False,
    keepelreg_flag: bool = False,
    keepallm3z_flag: bool = False,
    cleanall_flag: bool = False,
    cleansurfreg_flag: bool = False,
    cleanelreg_flag: bool = False,
    cleanvolreg_flag: bool = False,
    m3d_flag: bool = False,
    openmp: int | None = None,
    nolog_flag: bool = False,
    version_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> MriCvsRegisterOutputs:
    """
    Combined Volume and Surface Registration.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mov_subjid: FreeSurfer subject name as found in $SUBJECTS_DIR. This\
            identifies the subject that is to be moved / registered to the target.
        template_subjid: FreeSurfer subject name as found in $SUBJECTS_DIR (or\
            --templatedir). This identifies the subject that is to be kept fixed\
            (template).
        templatedir: Directory of the template subject's SUBJECTS_DIR if\
            different from that of the moving subject.
        mni_flag: Use the CVS atlas in MNI152 space as a target for\
            registration.
        outdir: Name of the output directory where all the registration results\
            are written.
        asegfname: Name of the segmentation volume used in volumetric\
            registration step. Do not use the file extension.
        voltype: Changes the input from norm.mgz to voltype.mgz.
        step1_flag: Only do step 1 (spherical registration).
        step2_flag: Only do step 2 (elastic registration).
        step3_flag: Only do step 3 (volumetric registration).
        noaseg_flag: Do not use aseg volumes in the volumetric registration\
            pipeline.
        nointensity_flag: Do not use intensity volumes in the volumetric\
            registration pipeline.
        hemi_flag: Run CVS registration only on one hemisphere.
        masktargethemi_flag: Use with --hemi when hemi is registered to full\
            brain target.
        maskmovinghemi_flag: Use with --hemi when full brain is registered to\
            single hemi target.
        nocleanup_flag: Do not delete temporary files.
        keepelreg_flag: Do not delete elastic registration outcomes.
        keepallm3z_flag: Do not delete m3z morph files computed during CVS.
        cleanall_flag: Overwrite / recompute all CVS-related morphs.
        cleansurfreg_flag: Overwrite/recompute CVS-related surface registration\
            morphs.
        cleanelreg_flag: Overwrite / recompute CVS-related elastic registration\
            morph.
        cleanvolreg_flag: Overwrite / recompute CVS-related volumetric morphs.
        m3d_flag: Use m3d instead of m3z for registration morphs.
        openmp: Assign the number of nodes for openmp runs.
        nolog_flag: Do not produce a log file.
        version_flag: Print version and exit.
        help_flag: Print help and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCvsRegisterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CVS_REGISTER_METADATA)
    cargs = []
    cargs.append("mri_cvs_register")
    cargs.extend([
        "--mov",
        mov_subjid
    ])
    if template_subjid is not None:
        cargs.extend([
            "--template",
            template_subjid
        ])
    if templatedir is not None:
        cargs.extend([
            "--templatedir",
            templatedir
        ])
    if mni_flag:
        cargs.append("--mni")
    if outdir is not None:
        cargs.extend([
            "--outdir",
            outdir
        ])
    if asegfname is not None:
        cargs.extend([
            "--asegfname",
            asegfname
        ])
    if voltype is not None:
        cargs.extend([
            "--voltype",
            voltype
        ])
    if step1_flag:
        cargs.append("--step1")
    if step2_flag:
        cargs.append("--step2")
    if step3_flag:
        cargs.append("--step3")
    if noaseg_flag:
        cargs.append("--noaseg")
    if nointensity_flag:
        cargs.append("--nointensity")
    if hemi_flag:
        cargs.append("--hemi")
    if masktargethemi_flag:
        cargs.append("--masktargethemi")
    if maskmovinghemi_flag:
        cargs.append("--maskmovinghemi")
    if nocleanup_flag:
        cargs.append("--nocleanup")
    if keepelreg_flag:
        cargs.append("--keepelreg")
    if keepallm3z_flag:
        cargs.append("--keepallm3z")
    if cleanall_flag:
        cargs.append("--cleanall")
    if cleansurfreg_flag:
        cargs.append("--cleansurfreg")
    if cleanelreg_flag:
        cargs.append("--cleanelreg")
    if cleanvolreg_flag:
        cargs.append("--cleanvolreg")
    if m3d_flag:
        cargs.append("--m3d")
    if openmp is not None:
        cargs.extend([
            "--openmp",
            str(openmp)
        ])
    if nolog_flag:
        cargs.append("--nolog")
    if version_flag:
        cargs.append("--version")
    if help_flag:
        cargs.append("--help")
    ret = MriCvsRegisterOutputs(
        root=execution.output_file("."),
        final_cvs_morph=execution.output_file(outdir + "/final_CVSmorph_toTEMPLATE.m3z") if (outdir is not None) else None,
        final_cvs_morphed_norm=execution.output_file(outdir + "/final_CVSmorphed_toTEMPLATE_norm.mgz") if (outdir is not None) else None,
        final_cvs_morphed_aseg=execution.output_file(outdir + "/final_CVSmorphed_toTEMPLATE_aseg.mgz") if (outdir is not None) else None,
        step1_morphed_aseg=execution.output_file(outdir + "/step1_CVSmorphed_toTEMPLATE_aseg.mgz") if (outdir is not None) else None,
        step1_morphed_norm=execution.output_file(outdir + "/step1_CVSmorphed_toTEMPLATE_norm.mgz") if (outdir is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CVS_REGISTER_METADATA",
    "MriCvsRegisterOutputs",
    "mri_cvs_register",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_EXVIVO_NORM_METADATA = Metadata(
    id="a88bc50afe4bde5d431a45ef570ba7c5a89a8c36.boutiques",
    name="mri_exvivo_norm",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriExvivoNormOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_exvivo_norm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume: OutputPathType
    """Output MRI volume"""


def mri_exvivo_norm(
    input_volume: InputPathType,
    output_volume: str,
    hemi: str,
    prediction_volume: str | None = None,
    normalized_volume: str | None = None,
    freeview: bool = False,
    normalize_output_mean: bool = False,
    write_normalization_rounds: bool = False,
    upper_threshold: float | None = None,
    bias_field_sigma: float | None = None,
    normalization_rounds: float | None = None,
    multichannel: bool = False,
    model_file: InputPathType | None = None,
    weights_file: InputPathType | None = None,
    gpu_number: float | None = None,
    runner: Runner | None = None,
) -> MriExvivoNormOutputs:
    """
    MRI Ex Vivo Normalization Tool.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: Input MRI volume.
        output_volume: Output MRI volume.
        hemi: Hemi to process.
        prediction_volume: Write prediction volume.
        normalized_volume: Write normalized volume.
        freeview: Bring up freeview to show results.
        normalize_output_mean: Normalize output mean to match input mean.
        write_normalization_rounds: Write normalization results after each\
            round.
        upper_threshold: Specify threshold to erase above.
        bias_field_sigma: Sigma to smooth bias field.
        normalization_rounds: Number of rounds of iterative normalization to\
            apply.
        multichannel: Specify that data has multiple channels.
        model_file: Use alternative model file.
        weights_file: Alternative weights filename.
        gpu_number: GPU number - if not supplied, CPU is used.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriExvivoNormOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_EXVIVO_NORM_METADATA)
    cargs = []
    cargs.append("mri_exvivo_norm")
    cargs.extend([
        "-i",
        execution.input_file(input_volume)
    ])
    cargs.extend([
        "-o",
        output_volume
    ])
    cargs.extend([
        "--hemi",
        hemi
    ])
    if prediction_volume is not None:
        cargs.extend([
            "--pred",
            prediction_volume
        ])
    if normalized_volume is not None:
        cargs.extend([
            "--norm",
            normalized_volume
        ])
    if freeview:
        cargs.append("--fv")
    if normalize_output_mean:
        cargs.append("--norm_mean")
    if write_normalization_rounds:
        cargs.append("--write_rounds")
    if upper_threshold is not None:
        cargs.extend([
            "--uthresh",
            str(upper_threshold)
        ])
    if bias_field_sigma is not None:
        cargs.extend([
            "--sigma",
            str(bias_field_sigma)
        ])
    if normalization_rounds is not None:
        cargs.extend([
            "--nrounds",
            str(normalization_rounds)
        ])
    if multichannel:
        cargs.append("--multichannel")
    if model_file is not None:
        cargs.extend([
            "--model",
            execution.input_file(model_file)
        ])
    if weights_file is not None:
        cargs.extend([
            "--wts",
            execution.input_file(weights_file)
        ])
    if gpu_number is not None:
        cargs.extend([
            "--gpu",
            str(gpu_number)
        ])
    ret = MriExvivoNormOutputs(
        root=execution.output_file("."),
        output_volume=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_EXVIVO_NORM_METADATA",
    "MriExvivoNormOutputs",
    "mri_exvivo_norm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

QUANTIFY_HASUBREGIONS_SH_METADATA = Metadata(
    id="df100705a41731ca1f463e42b7e69b52a1c9d0b4.boutiques",
    name="quantifyHAsubregions.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class QuantifyHasubregionsShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `quantify_hasubregions_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """File containing the quantification results of hippocampal subregions."""


def quantify_hasubregions_sh(
    prefix: str,
    suffix: str,
    output_file: str,
    subjects_directory: str | None = None,
    runner: Runner | None = None,
) -> QuantifyHasubregionsShOutputs:
    """
    Tool to quantify hippocampal subregions using FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        prefix: Prefix for the files to be processed.
        suffix: Suffix for the files to be processed.
        output_file: Output file name to store the results.
        subjects_directory: Directory containing the subject data. If not\
            provided, the current directory is used.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `QuantifyHasubregionsShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(QUANTIFY_HASUBREGIONS_SH_METADATA)
    cargs = []
    cargs.append("quantifyHAsubregions.sh")
    cargs.append(prefix)
    cargs.append(suffix)
    cargs.append(output_file)
    if subjects_directory is not None:
        cargs.append(subjects_directory)
    ret = QuantifyHasubregionsShOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "QUANTIFY_HASUBREGIONS_SH_METADATA",
    "QuantifyHasubregionsShOutputs",
    "quantify_hasubregions_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DCM2NIIX_METADATA = Metadata(
    id="15ccd57203c6c358b266b58ae05fde04fa3d5414.boutiques",
    name="dcm2niix",
    package="dcm2niix",
    container_image_tag="vnmd/dcm2niix_v1.0.20240202:20241125",
)


class Dcm2niixOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dcm2niix_(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dcm2niix_(
    input_dir: InputPathType,
    compression_level: float | None = None,
    adjacent: typing.Literal["y", "n"] | None = None,
    bids: typing.Literal["y", "n", "o"] | None = None,
    bids_anon: typing.Literal["y", "n"] | None = None,
    comment: str | None = None,
    depth: float | None = None,
    export_format: typing.Literal["y", "n", "o", "j", "b"] | None = None,
    filename: str | None = None,
    defaults: typing.Literal["y", "n", "o", "i"] | None = None,
    ignore_derived: typing.Literal["y", "n"] | None = None,
    scaling: typing.Literal["y", "n", "o"] | None = None,
    merge_2d: typing.Literal["n", "y", "0", "1", "2"] | None = None,
    series_number: str | None = None,
    output_dir: str | None = ".",
    philips_scaling: typing.Literal["y", "n"] | None = None,
    search_mode: typing.Literal["y", "l", "n"] | None = None,
    rename: typing.Literal["y", "n"] | None = None,
    single_file: typing.Literal["y", "n"] | None = None,
    update_check: bool = False,
    verbose: typing.Literal["0", "1", "2"] | None = None,
    conflict_behavior: float | None = None,
    crop_3d: typing.Literal["y", "n", "i"] | None = None,
    compression: typing.Literal["y", "o", "i", "n", "3"] | None = None,
    endian: typing.Literal["y", "n", "o"] | None = None,
    progress: typing.Literal["y", "n"] | None = None,
    ignore_trigger: bool = False,
    terse: bool = False,
    xml_: bool = False,
    runner: Runner | None = None,
) -> Dcm2niixOutputs:
    """
    Chris Rorden's dcm2niiX - DICOM to NIfTI converter. Converts DICOM files to
    NIfTI format with optional BIDS sidecar generation.
    
    Author: Chris Rorden
    
    Args:
        input_dir: Input folder containing DICOM files. Will be searched\
            recursively based on depth parameter.
        compression_level: gz compression level (1=fastest..9=smallest).
        adjacent: Adjacent DICOMs (images from same series always in same\
            folder) for faster conversion.
        bids: Generate BIDS sidecar JSON files (o=only: no NIfTI).
        bids_anon: Anonymize BIDS sidecar files by removing personal\
            information.
        comment: Comment stored in NIfTI aux_file (up to 24 characters).
        depth: Directory search depth for DICOM files in sub-folders.
        export_format: Export format: NRRD (y), MGH (o), JSON/JNIfTI (j), or\
            BJNIfTI (b).
        filename: Output filename template (%a=antenna, %b=basename,\
            %c=comments, %d=description, %e=echo number, %f=folder name,\
            %g=accession number, %i=ID of patient, %j=seriesInstanceUID,\
            %k=studyInstanceUID, %m=manufacturer, %n=name of patient,\
            %o=mediaObjectInstanceUID, %p=protocol, %r=instance number, %s=series\
            number, %t=time, %u=acquisition number, %v=vendor, %x=study ID;\
            %z=sequence name).
        defaults: Generate defaults file (o=only: reset and write defaults;\
            i=ignore: reset defaults).
        ignore_derived: Ignore derived, localizer and 2D images.
        scaling: Losslessly scale 16-bit integers (y=scale, n=no but\
            uint16->int16, o=original).
        merge_2d: Merge 2D slices from same series regardless of echo,\
            exposure, etc. (0=no, 1=yes, 2=auto).
        series_number: Only convert specified series CRC number (can be used up\
            to 16 times).
        output_dir: Output directory (omit to save to input folder).
        philips_scaling: Use Philips precise float (not display) scaling.
        search_mode: Search mode (y=show number of DICOMs, l=list DICOMs, n=no).
        rename: Rename instead of convert DICOMs.
        single_file: Single file mode, do not convert other images in folder.
        update_check: Check for newer versions.
        verbose: Verbose output (0=no, 1=yes, 2=logorrheic).
        conflict_behavior: Write behavior for name conflicts (0=skip,\
            1=overwrite, 2=add suffix).
        crop_3d: Crop 3D acquisitions (i=ignore: neither crop nor rotate).
        compression: gz compress images (y=pigz, o=optimal pigz,\
            i=internal:zlib, n=no, 3=no,3D).
        endian: Byte order (y=big-end, n=little-end, o=optimal/native).
        progress: Slicer format progress information.
        ignore_trigger: Disregard values in 0018,1060 and 0020,9153.
        terse: Omit filename post-fixes (can cause overwrites).
        xml_: Output Slicer format features.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dcm2niixOutputs`).
    """
    if compression_level is not None and not (1 <= compression_level <= 9): 
        raise ValueError(f"'compression_level' must be between 1 <= x <= 9 but was {compression_level}")
    if depth is not None and not (0 <= depth <= 9): 
        raise ValueError(f"'depth' must be between 0 <= x <= 9 but was {depth}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(DCM2NIIX_METADATA)
    cargs = []
    cargs.append("dcm2niix")
    if compression_level is not None:
        cargs.append("-" + str(compression_level))
    if adjacent is not None:
        cargs.extend([
            "-a",
            adjacent
        ])
    if bids is not None:
        cargs.extend([
            "-b",
            bids
        ])
    if bids_anon is not None:
        cargs.extend([
            "-ba",
            bids_anon
        ])
    if comment is not None:
        cargs.extend([
            "-c",
            comment
        ])
    if depth is not None:
        cargs.extend([
            "-d",
            str(depth)
        ])
    if export_format is not None:
        cargs.extend([
            "-e",
            export_format
        ])
    if filename is not None:
        cargs.extend([
            "-f",
            filename
        ])
    if defaults is not None:
        cargs.extend([
            "-g",
            defaults
        ])
    if ignore_derived is not None:
        cargs.extend([
            "-i",
            ignore_derived
        ])
    if scaling is not None:
        cargs.extend([
            "-l",
            scaling
        ])
    if merge_2d is not None:
        cargs.extend([
            "-m",
            merge_2d
        ])
    if series_number is not None:
        cargs.extend([
            "-n",
            series_number
        ])
    if output_dir is not None:
        cargs.extend([
            "-o",
            output_dir
        ])
    if philips_scaling is not None:
        cargs.extend([
            "-p",
            philips_scaling
        ])
    if search_mode is not None:
        cargs.extend([
            "-q",
            search_mode
        ])
    if rename is not None:
        cargs.extend([
            "-r",
            rename
        ])
    if single_file is not None:
        cargs.extend([
            "-s",
            single_file
        ])
    if update_check:
        cargs.append("-u")
    if verbose is not None:
        cargs.extend([
            "-v",
            verbose
        ])
    if conflict_behavior is not None:
        cargs.extend([
            "-w",
            str(conflict_behavior)
        ])
    if crop_3d is not None:
        cargs.extend([
            "-x",
            crop_3d
        ])
    if compression is not None:
        cargs.extend([
            "-z",
            compression
        ])
    if endian is not None:
        cargs.extend([
            "--big-endian",
            endian
        ])
    if progress is not None:
        cargs.extend([
            "--progress",
            progress
        ])
    if ignore_trigger:
        cargs.append("--ignore_trigger_times")
    if terse:
        cargs.append("--terse")
    if xml_:
        cargs.append("--xml")
    cargs.append(execution.input_file(input_dir))
    ret = Dcm2niixOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DCM2NIIX_METADATA",
    "Dcm2niixOutputs",
    "dcm2niix_",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CLUSTER_METADATA = Metadata(
    id="f9c277296bff6829746b7b450932118c7250ea4d.boutiques",
    name="cluster",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class ClusterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cluster(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    index_file: OutputPathType
    """Output of cluster index (in size order)."""
    localmax_txt_file: OutputPathType
    """Local maxima text file."""
    localmax_vol_file: OutputPathType
    """Output of local maxima volume."""
    max_file: OutputPathType
    """Filename for output of max image."""
    mean_file: OutputPathType
    """Filename for output of mean image."""
    pval_file: OutputPathType
    """Filename for image output of log pvals."""
    size_file: OutputPathType
    """Filename for output of size image."""
    threshold_file: OutputPathType
    """Thresholded image."""


def cluster(
    in_file: InputPathType,
    threshold: float,
    connectivity: int | None = None,
    cope_file: InputPathType | None = None,
    dlh: float | None = None,
    find_min: bool = False,
    fractional: bool = False,
    minclustersize: bool = False,
    no_table: bool = False,
    num_maxima: int | None = None,
    out_index_file_2: InputPathType | None = None,
    out_localmax_txt_file_2: InputPathType | None = None,
    out_localmax_vol_file_2: InputPathType | None = None,
    out_max_file_2: InputPathType | None = None,
    out_mean_file_2: InputPathType | None = None,
    out_pval_file_2: InputPathType | None = None,
    out_size_file_2: InputPathType | None = None,
    out_threshold_file_2: InputPathType | None = None,
    output_type: typing.Literal["NIFTI", "NIFTI_PAIR", "NIFTI_GZ", "NIFTI_PAIR_GZ"] | None = None,
    peak_distance: float | None = None,
    pthreshold: float | None = None,
    std_space_file: InputPathType | None = None,
    use_mm: bool = False,
    volume: int | None = None,
    warpfield_file: InputPathType | None = None,
    xfm_file: InputPathType | None = None,
    runner: Runner | None = None,
) -> ClusterOutputs:
    """
    Uses FSL cluster to perform clustering on statistical output.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        in_file: Input volume.
        threshold: Threshold for input volume.
        connectivity: The connectivity of voxels (default 26).
        cope_file: Cope volume.
        dlh: Smoothness estimate = sqrt(det(lambda)).
        find_min: Find minima instead of maxima.
        fractional: Interprets the threshold as a fraction of the robust range.
        minclustersize: Prints out minimum significant cluster size.
        no_table: Suppresses printing of the table info.
        num_maxima: No of local maxima to report.
        out_index_file_2: A boolean or file. Output of cluster index (in size\
            order).
        out_localmax_txt_file_2: A boolean or file. Local maxima text file.
        out_localmax_vol_file_2: A boolean or file. Output of local maxima\
            volume.
        out_max_file_2: A boolean or file. Filename for output of max image.
        out_mean_file_2: A boolean or file. Filename for output of mean image.
        out_pval_file_2: A boolean or file. Filename for image output of log\
            pvals.
        out_size_file_2: A boolean or file. Filename for output of size image.
        out_threshold_file_2: A boolean or file. Thresholded image.
        output_type: 'nifti' or 'nifti_pair' or 'nifti_gz' or 'nifti_pair_gz'.\
            Fsl output type.
        peak_distance: Minimum distance between local maxima/minima, in mm\
            (default 0).
        pthreshold: P-threshold for clusters.
        std_space_file: Filename for standard-space volume.
        use_mm: Use mm, not voxel, coordinates.
        volume: Number of voxels in the mask.
        warpfield_file: File containing warpfield.
        xfm_file: Filename for linear: input->standard-space transform.\
            non-linear: input->highres transform.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ClusterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CLUSTER_METADATA)
    cargs = []
    cargs.append("cluster")
    if connectivity is not None:
        cargs.append("--connectivity=" + str(connectivity))
    if cope_file is not None:
        cargs.append("--cope=" + execution.input_file(cope_file))
    if dlh is not None:
        cargs.append("--dlh=" + str(dlh))
    if find_min:
        cargs.append("--min")
    if fractional:
        cargs.append("--fractional")
    cargs.append("--in=" + execution.input_file(in_file))
    if minclustersize:
        cargs.append("--minclustersize")
    if no_table:
        cargs.append("--no_table")
    if num_maxima is not None:
        cargs.append("--num=" + str(num_maxima))
    if out_index_file_2 is not None:
        cargs.append("--oindex=" + execution.input_file(out_index_file_2))
    if out_localmax_txt_file_2 is not None:
        cargs.append("--olmax=" + execution.input_file(out_localmax_txt_file_2))
    if out_localmax_vol_file_2 is not None:
        cargs.append("--olmaxim=" + execution.input_file(out_localmax_vol_file_2))
    if out_max_file_2 is not None:
        cargs.append("--omax=" + execution.input_file(out_max_file_2))
    if out_mean_file_2 is not None:
        cargs.append("--omean=" + execution.input_file(out_mean_file_2))
    if out_pval_file_2 is not None:
        cargs.append("--opvals=" + execution.input_file(out_pval_file_2))
    if out_size_file_2 is not None:
        cargs.append("--osize=" + execution.input_file(out_size_file_2))
    if out_threshold_file_2 is not None:
        cargs.append("--othresh=" + execution.input_file(out_threshold_file_2))
    if output_type is not None:
        cargs.append(output_type)
    if peak_distance is not None:
        cargs.append("--peakdist=" + str(peak_distance))
    if pthreshold is not None:
        cargs.append("--pthresh=" + str(pthreshold))
    if std_space_file is not None:
        cargs.append("--stdvol=" + execution.input_file(std_space_file))
    cargs.append("--thresh=" + str(threshold))
    if use_mm:
        cargs.append("--mm")
    if volume is not None:
        cargs.append("--volume=" + str(volume))
    if warpfield_file is not None:
        cargs.append("--warpvol=" + execution.input_file(warpfield_file))
    if xfm_file is not None:
        cargs.append("--xfm=" + execution.input_file(xfm_file))
    ret = ClusterOutputs(
        root=execution.output_file("."),
        index_file=execution.output_file("index_file"),
        localmax_txt_file=execution.output_file("localmax_txt_file"),
        localmax_vol_file=execution.output_file("localmax_vol_file"),
        max_file=execution.output_file("max_file"),
        mean_file=execution.output_file("mean_file"),
        pval_file=execution.output_file("pval_file"),
        size_file=execution.output_file("size_file"),
        threshold_file=execution.output_file("threshold_file"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CLUSTER_METADATA",
    "ClusterOutputs",
    "cluster",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

LESION_FILLING_METADATA = Metadata(
    id="e67424ecd9a0c5e13dd9b069bccb2e63e4348726.boutiques",
    name="LesionFilling",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class LesionFillingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `lesion_filling(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    lesion_filled_output: OutputPathType
    """Output image with filled lesions."""


def lesion_filling(
    image_dimension: int,
    t1_image: InputPathType,
    lesion_mask: InputPathType,
    output_lesion_filled: str,
    runner: Runner | None = None,
) -> LesionFillingOutputs:
    """
    A tool for filling lesions in T1 images using a mask.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimensionality of the image (e.g., 2, 3).
        t1_image: Path to the T1 image file.
        lesion_mask: Path to the lesion mask image file.
        output_lesion_filled: Path for the output file with lesions filled.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LesionFillingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LESION_FILLING_METADATA)
    cargs = []
    cargs.append("LesionFilling")
    cargs.append(str(image_dimension))
    cargs.append(execution.input_file(t1_image))
    cargs.append(execution.input_file(lesion_mask))
    cargs.append(output_lesion_filled)
    ret = LesionFillingOutputs(
        root=execution.output_file("."),
        lesion_filled_output=execution.output_file(output_lesion_filled),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "LESION_FILLING_METADATA",
    "LesionFillingOutputs",
    "lesion_filling",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_DEFECTS_POINTSET_METADATA = Metadata(
    id="83e333d1259fc372d991cc353b27693763bfdd37.boutiques",
    name="mris_defects_pointset",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisDefectsPointsetOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_defects_pointset(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    pointset_output: OutputPathType
    """Output pointset file containing locations of topological defects"""


def mris_defects_pointset(
    surface: InputPathType,
    defects: InputPathType,
    out: InputPathType,
    label: InputPathType | None = None,
    control: bool = False,
    runner: Runner | None = None,
) -> MrisDefectsPointsetOutputs:
    """
    Produces a pointset file containing the locations of each topological defect in
    a surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surface: Input surface.
        defects: Input defect label (must match the surface dimensions).
        out: Output pointset file (json).
        label: Restrict pointset to this label (must be in input surface space).
        control: Save output in old control point format (v6 compatible).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisDefectsPointsetOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_DEFECTS_POINTSET_METADATA)
    cargs = []
    cargs.append("mris_defects_pointset")
    cargs.extend([
        "--surf",
        execution.input_file(surface)
    ])
    cargs.extend([
        "--defects",
        execution.input_file(defects)
    ])
    cargs.extend([
        "--out",
        execution.input_file(out)
    ])
    if label is not None:
        cargs.extend([
            "--label",
            execution.input_file(label)
        ])
    if control:
        cargs.append("--control")
    ret = MrisDefectsPointsetOutputs(
        root=execution.output_file("."),
        pointset_output=execution.output_file(pathlib.Path(out).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_DEFECTS_POINTSET_METADATA",
    "MrisDefectsPointsetOutputs",
    "mris_defects_pointset",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TBSS_3_POSTREG_METADATA = Metadata(
    id="8629f29c3ee03810baedd803bec842f121834cff.boutiques",
    name="tbss_3_postreg",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class Tbss3PostregOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tbss_3_postreg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tbss_3_postreg(
    derive_mean_from_study: bool = False,
    use_fmrib58: bool = False,
    runner: Runner | None = None,
) -> Tbss3PostregOutputs:
    """
    TBSS post-registration processing.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        derive_mean_from_study: Derive mean_FA and mean_FA_skeleton from mean\
            of all subjects in study (recommended).
        use_fmrib58: Use FMRIB58_FA and its skeleton instead of study-derived\
            mean and skeleton.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tbss3PostregOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TBSS_3_POSTREG_METADATA)
    cargs = []
    cargs.append("tbss_3_postreg")
    if derive_mean_from_study:
        cargs.append("-S")
    if use_fmrib58:
        cargs.append("-T")
    ret = Tbss3PostregOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TBSS_3_POSTREG_METADATA",
    "Tbss3PostregOutputs",
    "tbss_3_postreg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DICOM_RENAME_METADATA = Metadata(
    id="2eb87845c3a6be71bcd6bb57089f0cd879da57d4.boutiques",
    name="dicom-rename",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class DicomRenameOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dicom_rename(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    renamed_dicom: OutputPathType
    """Renamed output DICOM file with series and image numbers."""


def dicom_rename(
    input_files: list[InputPathType],
    output_base: str,
    runner: Runner | None = None,
) -> DicomRenameOutputs:
    """
    Copies dicom file(s) to new path with more meaningful names.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_files: Input DICOM files to be renamed.
        output_base: Base name for output files that includes series and image\
            numbers.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DicomRenameOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DICOM_RENAME_METADATA)
    cargs = []
    cargs.extend([
        "-rename",
        "dicom" + "".join([execution.input_file(f) for f in input_files])
    ])
    cargs.extend([
        "--o",
        output_base
    ])
    ret = DicomRenameOutputs(
        root=execution.output_file("."),
        renamed_dicom=execution.output_file(output_base + "-SSS-IIIII.dcm"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DICOM_RENAME_METADATA",
    "DicomRenameOutputs",
    "dicom_rename",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FAT_PROC_IMIT2W_FROM_T1W_METADATA = Metadata(
    id="6e7211eeaf18937b51ec873d5287062ff3f44746.boutiques",
    name="fat_proc_imit2w_from_t1w",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FatProcImit2wFromT1wOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fat_proc_imit2w_from_t1w(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    t2w_contrast_volume: OutputPathType
    """Volume with T2w-like tissue contrast from T1w"""
    cleaned_t1w_volume: OutputPathType
    """Cleaned/processed version of the input T1w volume with scaled skull and
    noise outside the brain"""
    skull_stripped_t1w: OutputPathType
    """Skull-stripped version of the T1w volume"""
    qc_images: OutputPathType
    """QC images of the skull-stripped T1w volume and the final imitation-T2w
    volume"""


def fat_proc_imit2w_from_t1w(
    t1_file: InputPathType,
    prefix: str,
    workdir: str | None = None,
    mask: InputPathType | None = None,
    ss_blur_fwhm: float | None = None,
    no_clean: bool = False,
    no_qc_view: bool = False,
    qc_prefix: str | None = None,
    runner: Runner | None = None,
) -> FatProcImit2wFromT1wOutputs:
    """
    Process T1w anatomical images to generate an imitation T2w-contrast image.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        t1_file: Full name of the input T1w volume.
        prefix: Output prefix for files and snapshots.
        workdir: Specify a working directory, which can be removed (default:\
            __WORKING_imit2w_from_t1w).
        mask: Optional input of a pre-skullstripped T1_FILE (either mask or\
            skull-stripped volume).
        ss_blur_fwhm: Optional, add in blurring during the 3dSkullStrip part\
            (in mm, default: 2 FWHM).
        no_clean: Optional switch to NOT remove working directory\
            '__WORKING_imit2w_from_t1w' (default: remove working dir).
        no_qc_view: Turn off the automatic creation of QC montages (default:\
            on).
        qc_prefix: Change the prefix of the QC images (default: use prefix of\
            volumes).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FatProcImit2wFromT1wOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FAT_PROC_IMIT2W_FROM_T1W_METADATA)
    cargs = []
    cargs.append("fat_proc_imit2w_from_t1w")
    cargs.extend([
        "-inset",
        execution.input_file(t1_file)
    ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    if workdir is not None:
        cargs.extend([
            "-workdir",
            workdir
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if ss_blur_fwhm is not None:
        cargs.extend([
            "-ss_blur_fwhm",
            str(ss_blur_fwhm)
        ])
    if no_clean:
        cargs.append("-no_clean")
    if no_qc_view:
        cargs.append("-no_qc_view")
    if qc_prefix is not None:
        cargs.extend([
            "-qc_prefix",
            qc_prefix
        ])
    ret = FatProcImit2wFromT1wOutputs(
        root=execution.output_file("."),
        t2w_contrast_volume=execution.output_file(prefix + ".nii.gz"),
        cleaned_t1w_volume=execution.output_file(prefix + "_orig.nii.gz"),
        skull_stripped_t1w=execution.output_file(prefix + "_orig_ss.nii.gz"),
        qc_images=execution.output_file(prefix + "_qc*.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FAT_PROC_IMIT2W_FROM_T1W_METADATA",
    "FatProcImit2wFromT1wOutputs",
    "fat_proc_imit2w_from_t1w",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_DWITO_DT_METADATA = Metadata(
    id="edef0edb2012159a76d51a09912a5bd6c5660ab2.boutiques",
    name="3dDWItoDT",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dDwitoDtOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_dwito_dt(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType
    """Output dataset"""


def v_3d_dwito_dt(
    gradient_file: InputPathType,
    dataset: InputPathType,
    runner: Runner | None = None,
) -> V3dDwitoDtOutputs:
    """
    Computes 6 principal direction tensors from multiple gradient vectors and
    corresponding DTI image volumes.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        gradient_file: A 1D file of the gradient vectors with lines of ASCII\
            floats (Gxi, Gyi, Gzi) or a 1D file of b-matrix elements.
        dataset: A 3D bucket dataset with Np+1 sub-briks where the first\
            sub-brik is the volume acquired with no diffusion weighting.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dDwitoDtOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_DWITO_DT_METADATA)
    cargs = []
    cargs.append("3dDWItoDT")
    cargs.append("[OPTIONS]")
    cargs.append(execution.input_file(gradient_file))
    cargs.append(execution.input_file(dataset))
    ret = V3dDwitoDtOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file("<PREFIX>.*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dDwitoDtOutputs",
    "V_3D_DWITO_DT_METADATA",
    "v_3d_dwito_dt",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SPMREGISTER_METADATA = Metadata(
    id="f04847924393c8dd412a6711f383105c39a90534.boutiques",
    name="spmregister",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class SpmregisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `spmregister(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    registration_file: OutputPathType
    """Output registration file."""
    resampled_mov: OutputPathType | None
    """Resampled mov volume."""


def spmregister(
    subjid: str,
    mov: str,
    reg: str,
    frame: float | None = None,
    mid_frame: bool = False,
    template_out: str | None = None,
    fsvol: str | None = None,
    force_ras: bool = False,
    outvol: str | None = None,
    tmpdir: str | None = None,
    nocleanup: bool = False,
    version: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> SpmregisterOutputs:
    """
    Registers a volume to its FreeSurfer anatomical using SPM's spm_coreg.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjid: Id of the subject as found in SUBJECTS_DIR. This is converted\
            to analyze using mri_convert.
        mov: Volume identifier of the movable volume. Must be specified in a\
            way suitable for mri_convert.
        reg: Output registration file. Maps RAS in the reference to RAS in the\
            movable.
        frame: Use something other than the first frame. Specify the frame\
            number you want.
        mid_frame: Use the middle frame of the mov volume as the template.
        template_out: Save the mov template when template is something other\
            than the first frame.
        fsvol: Use FreeSurfer volid (default brainmask).
        force_ras: Force input geometry to be RAS.
        outvol: Resample mov and save as outvol.
        tmpdir: Temporary directory (implies --nocleanup).
        nocleanup: Do not delete temporary files.
        version: Print version and exit.
        help_: Print help and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpmregisterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SPMREGISTER_METADATA)
    cargs = []
    cargs.append("spmregister")
    cargs.extend([
        "--s",
        subjid
    ])
    cargs.extend([
        "--mov",
        mov
    ])
    cargs.extend([
        "--reg",
        reg
    ])
    if frame is not None:
        cargs.extend([
            "--frame",
            str(frame)
        ])
    if mid_frame:
        cargs.append("--mid-frame")
    if template_out is not None:
        cargs.extend([
            "--template-out",
            template_out
        ])
    if fsvol is not None:
        cargs.extend([
            "--fsvol",
            fsvol
        ])
    if force_ras:
        cargs.append("--force-ras")
    if outvol is not None:
        cargs.extend([
            "--o",
            outvol
        ])
    if tmpdir is not None:
        cargs.extend([
            "--tmp",
            tmpdir
        ])
    if nocleanup:
        cargs.append("--nocleanup")
    if version:
        cargs.append("--version")
    if help_:
        cargs.append("--help")
    ret = SpmregisterOutputs(
        root=execution.output_file("."),
        registration_file=execution.output_file("register.dat"),
        resampled_mov=execution.output_file(outvol) if (outvol is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SPMREGISTER_METADATA",
    "SpmregisterOutputs",
    "spmregister",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_STOPMASK_METADATA = Metadata(
    id="e7f2a3c2cfe3f7fafce248b01fa340177fa54cc2.boutiques",
    name="mri_stopmask",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriStopmaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_stopmask(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    generated_mask: OutputPathType
    """The generated stop mask output file"""


def mri_stopmask(
    output_mask: str,
    filled: list[InputPathType],
    aseg_presurf: InputPathType,
    lateral_ventricles: bool = False,
    wmsa: float | None = None,
    wm_voxels: InputPathType | None = None,
    brain_final_surfs: InputPathType | None = None,
    no_filled: bool = False,
    no_lv: bool = False,
    no_wmsa: bool = False,
    no_wm: bool = False,
    no_bfs: bool = False,
    runner: Runner | None = None,
) -> MriStopmaskOutputs:
    """
    This program creates a mask used to stop the search for the maximum gradient in
    mris_place_surface, preventing the surface from wandering into areas it should
    not.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_mask: Output stop mask in volume format.
        filled: Include voxels edited fill voxels that are set in the\
            filled.mgz.
        aseg_presurf: Used with --lv and/or --wmsa; Note: must be\
            aseg.presurf.mgz, not aseg.mgz.
        lateral_ventricles: Add lateral ventricles and choroid plexus to the\
            mask (needs --aseg).
        wmsa: Add WM hypointensities to the mask (needs --aseg); erode by given\
            distance away from any adjacent cortex.
        wm_voxels: Include voxels that =255 from wm.mgz.
        brain_final_surfs: Include voxels that =255 from brain.finalsurfs.mgz.
        no_filled: Turns off --filled option.
        no_lv: Turns off --lv option.
        no_wmsa: Turns off --wmsa option.
        no_wm: Turns off --wm option.
        no_bfs: Turns off --bfs option.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriStopmaskOutputs`).
    """
    if not (len(filled) <= 2): 
        raise ValueError(f"Length of 'filled' must be less than 2 but was {len(filled)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_STOPMASK_METADATA)
    cargs = []
    cargs.append("mri_stopmask")
    cargs.extend([
        "--o",
        output_mask
    ])
    cargs.extend([
        "--filled",
        *[execution.input_file(f) for f in filled]
    ])
    cargs.extend([
        "--aseg",
        execution.input_file(aseg_presurf)
    ])
    if lateral_ventricles:
        cargs.append("--lv")
    if wmsa is not None:
        cargs.extend([
            "--wmsa",
            str(wmsa)
        ])
    if wm_voxels is not None:
        cargs.extend([
            "--wm",
            execution.input_file(wm_voxels)
        ])
    if brain_final_surfs is not None:
        cargs.extend([
            "--bfs",
            execution.input_file(brain_final_surfs)
        ])
    if no_filled:
        cargs.append("--no-filled")
    if no_lv:
        cargs.append("--no-lv")
    if no_wmsa:
        cargs.append("--no-wmsa")
    if no_wm:
        cargs.append("--no-wm")
    if no_bfs:
        cargs.append("--no-bfs")
    ret = MriStopmaskOutputs(
        root=execution.output_file("."),
        generated_mask=execution.output_file(output_mask),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_STOPMASK_METADATA",
    "MriStopmaskOutputs",
    "mri_stopmask",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SAMSEG2RECON_METADATA = Metadata(
    id="d187c3e2881ef4e68bbc4c2919f41e8e9ad94cca.boutiques",
    name="samseg2recon",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Samseg2reconOutputs(typing.NamedTuple):
    """
    Output object returned when calling `samseg2recon(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    filled_mgz: OutputPathType
    """Filled MRI volume for the subject, if --fill is used."""
    norm_mgz: OutputPathType
    """Normalized MRI volume created from brain.mgz if --normalization2 is
    used."""
    orig_mgz: OutputPathType
    """Original MRI conformed volume."""


def samseg2recon(
    subject: str,
    samseg_dir: str | None = None,
    no_cc: bool = False,
    fill: bool = False,
    normalization2: bool = False,
    uchar: bool = False,
    no_keep_exc: bool = False,
    long_tp: float | None = None,
    base: bool = False,
    mask_file: InputPathType | None = None,
    from_recon_all: bool = False,
    force_update: bool = False,
    runner: Runner | None = None,
) -> Samseg2reconOutputs:
    """
    Creates and populates a subjects directory for use with recon-all from SAMSEG
    outputs.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: Subject identifier.
        samseg_dir: Output directory from samseg or samseg-long. Defaults to\
            subject/mri/samseg if not supplied.
        no_cc: Do not do corpus callosum segmentation.
        fill: Use samseg to create the filled.mgz used for tessellation.
        normalization2: Seg brain.mgz to norm.mgz.
        uchar: Convert to uchar.
        no_keep_exc: Do not keep extracerebral segmentations.
        long_tp: Process specific time point (TP) number from samsegdir.
        base: Process base, will find folder called base in samsegdir.
        mask_file: Use provided mask as brainmask instead of computing from seg.
        from_recon_all: Indicates execution from recon-all, preventing\
            overwrite of rawavg.mgz and orig.mgz.
        force_update: Force update of the subject directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Samseg2reconOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SAMSEG2RECON_METADATA)
    cargs = []
    cargs.append("samseg2recon")
    cargs.extend([
        "--s",
        subject
    ])
    if samseg_dir is not None:
        cargs.extend([
            "--samseg",
            samseg_dir
        ])
    if no_cc:
        cargs.append("--no-cc")
    if fill:
        cargs.append("--fill")
    if normalization2:
        cargs.append("--normalization2")
    if uchar:
        cargs.append("--uchar")
    if no_keep_exc:
        cargs.append("--no-keep-exc")
    if long_tp is not None:
        cargs.extend([
            "--long",
            str(long_tp)
        ])
    if base:
        cargs.append("--base")
    if mask_file is not None:
        cargs.extend([
            "--m",
            execution.input_file(mask_file)
        ])
    if from_recon_all:
        cargs.append("--from-recon-all")
    if force_update:
        cargs.append("--force-update")
    ret = Samseg2reconOutputs(
        root=execution.output_file("."),
        filled_mgz=execution.output_file(subject + "/mri/filled.mgz"),
        norm_mgz=execution.output_file(subject + "/mri/norm.mgz"),
        orig_mgz=execution.output_file(subject + "/mri/orig/001.mgz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SAMSEG2RECON_METADATA",
    "Samseg2reconOutputs",
    "samseg2recon",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_TFITTER_METADATA = Metadata(
    id="8451a7cd8aa8de8047fb4bbe324982a2fbf6f94a.boutiques",
    name="3dTfitter",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dTfitterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_tfitter(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_prefix: OutputPathType | None
    """Output dataset for LHS parameters."""
    fitted_time_series: OutputPathType | None
    """Output fitted time series dataset."""
    error_sums: OutputPathType | None
    """Output error sums dataset."""


def v_3d_tfitter(
    rhs: str,
    lhs: list[str] | None = None,
    polort: float | None = None,
    vthr: float | None = None,
    faltung: list[str] | None = None,
    lsqfit: bool = False,
    l1fit: bool = False,
    l2lasso: list[str] | None = None,
    lasso_centro_block: list[str] | None = None,
    l2sqrtlasso: list[str] | None = None,
    consign: list[str] | None = None,
    cons_fal: str | None = None,
    prefix: str | None = None,
    label: list[str] | None = None,
    fitts: str | None = None,
    errsum: str | None = None,
    mask: str | None = None,
    quiet: bool = False,
    runner: Runner | None = None,
) -> V3dTfitterOutputs:
    """
    * At each voxel, assembles and solves a set of linear equations.
    ++ The matrix at each voxel may be the same or may be different.
    ++ This flexibility (for voxel-wise regressors) is one feature
    that makes 3dTfitter different from 3dDeconvolve.
    ++ Another distinguishing feature is that 3dTfitter allows for
    L2, L1, and L2+L1 (LASSO) regression solvers, and allows you
    to impose sign constraints on the solution parameters.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        rhs: Specifies the right-hand-side 3D+time dataset. ('rset' can also be\
            a 1D file with 1 column).
        lhs: Specifies a column (or columns) of the left-hand-side matrix. More\
            than one 'lset' can follow the '-LHS' option.
        polort: Add 'p+1' Legendre polynomial columns to the LHS matrix.
        vthr: The value 'v' (between 0.0 and 0.09, inclusive) defines the\
            threshold below which LHS vectors will be omitted from the regression\
            analysis.
        faltung: Specifies a convolution (German: Faltung) model to be added to\
            the LHS matrix. Followed by four arguments: 'fset', 'fpre', 'pen',\
            'fac'.
        lsqfit: Solve equations via least squares [the default method].
        l1fit: Solve equations via least sum of absolute residuals.
        l2lasso: Solve equations via least squares with a LASSO (L1) penalty on\
            the coefficients. Followed by 'lam' and optional 'i j k ...'.
        lasso_centro_block: Defines a block of coefficients that will be\
            penalized together.
        l2sqrtlasso: Similar to '-l2lasso', but aims to minimize\
            sqrt(Q2)+lam*L1.
        consign: Indicates that the sign of some output LHS parameters should\
            be constrained in the solution.
        cons_fal: Constrain the deconvolution time series from '-FALTUNG' to be\
            positive if 'c' is '+' or to be negative if 'c' is '-'.
        prefix: Prefix for the output dataset (LHS parameters) filename.
        label: Specifies sub-brick labels in the output LHS parameter dataset.
        fitts: Prefix filename for the output fitted time series dataset.
        errsum: Prefix filename for the error sums dataset.
        mask: Read in dataset 'ms' to use as a mask; only voxels with nonzero\
            values in the mask will be processed.
        quiet: Don't print progress report messages.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dTfitterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_TFITTER_METADATA)
    cargs = []
    cargs.append("3dTfitter")
    cargs.extend([
        "-RHS",
        rhs
    ])
    if lhs is not None:
        cargs.extend([
            "-LHS",
            *lhs
        ])
    if polort is not None:
        cargs.extend([
            "-polort",
            str(polort)
        ])
    if vthr is not None:
        cargs.extend([
            "-vthr",
            str(vthr)
        ])
    if faltung is not None:
        cargs.extend([
            "-FALTUNG",
            *faltung
        ])
    if lsqfit:
        cargs.append("-lsqfit")
    if l1fit:
        cargs.append("-l1fit")
    if l2lasso is not None:
        cargs.extend([
            "-l2lasso",
            *l2lasso
        ])
    if lasso_centro_block is not None:
        cargs.extend([
            "-lasso_centro_block",
            *lasso_centro_block
        ])
    if l2sqrtlasso is not None:
        cargs.extend([
            "-l2sqrtlasso",
            *l2sqrtlasso
        ])
    if consign is not None:
        cargs.extend([
            "-consign",
            *consign
        ])
    if cons_fal is not None:
        cargs.extend([
            "-consFAL",
            cons_fal
        ])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if label is not None:
        cargs.extend([
            "-label",
            *label
        ])
    if fitts is not None:
        cargs.extend([
            "-fitts",
            fitts
        ])
    if errsum is not None:
        cargs.extend([
            "-errsum",
            errsum
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            mask
        ])
    if quiet:
        cargs.append("-quiet")
    ret = V3dTfitterOutputs(
        root=execution.output_file("."),
        output_prefix=execution.output_file(prefix + ".nii.gz") if (prefix is not None) else None,
        fitted_time_series=execution.output_file(fitts + ".nii.gz") if (fitts is not None) else None,
        error_sums=execution.output_file(errsum + ".nii.gz") if (errsum is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dTfitterOutputs",
    "V_3D_TFITTER_METADATA",
    "v_3d_tfitter",
]
